from __future__ import unicode_literals

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DataElement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('error', models.FloatField(null=True)),
                ('int_data', models.IntegerField(null=True)),
                ('float_data', models.FloatField(null=True)),
                ('char_data', models.CharField(blank=True, max_length=200, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='DataField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field_en', models.CharField(blank=True, max_length=100)),
                ('field_longname', models.CharField(blank=True, max_length=400)),
                ('field_name', models.CharField(blank=True, max_length=50)),
                ('field_type', models.CharField(choices=[(b'I', b'integer'), (b'F', b'floating point'), (b'C', b'string')], default=b'C', max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='Dataset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('url', models.URLField(blank=True, max_length=300)),
                ('cached', models.DateTimeField(blank=True, null=True)),
                ('cache_max_age', models.IntegerField(default=1, verbose_name=b'age when cache should be replaced in days')),
                ('remote_id_field', models.CharField(blank=True, default=b'id', max_length=50, verbose_name=b'column name of key field on the remote server')),
                ('name_field', models.CharField(default=b'name', max_length=50)),
                ('lat_field', models.CharField(default=b'latitude', max_length=50)),
                ('lon_field', models.CharField(default=b'longitude', max_length=50)),
                ('street_field', models.CharField(default=b'street', max_length=50)),
                ('city_field', models.CharField(default=b'city', max_length=50)),
                ('state_field', models.CharField(default=b'state', max_length=50)),
                ('zipcode_field', models.CharField(default=b'zip', max_length=50)),
                ('county_field', models.CharField(default=b'county', max_length=50)),
                ('field1_en', models.CharField(blank=True, max_length=150)),
                ('field1_name', models.CharField(blank=True, max_length=50)),
                ('field2_en', models.CharField(blank=True, max_length=150)),
                ('field2_name', models.CharField(blank=True, max_length=50)),
                ('field3_en', models.CharField(blank=True, max_length=150)),
                ('field3_name', models.CharField(blank=True, max_length=50)),
                ('needs_geocoding', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='MapElement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('remote_id', models.CharField(blank=True, max_length=50, null=True)),
                ('name', models.CharField(max_length=150)),
                ('point', django.contrib.gis.db.models.fields.PointField(blank=True, null=True, srid=4326)),
            ],
        ),
        migrations.CreateModel(
            name='Observation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time', models.DateTimeField(null=True)),
                ('accuracy', models.FloatField(null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ObservationValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=50)),
                ('value', models.FloatField()),
                ('observation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='values', to='gis_csdt.Observation')),
            ],
        ),
        migrations.CreateModel(
            name='Sensor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('sensor_type', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tag', models.CharField(max_length=100)),
                ('approved', models.BooleanField(default=False)),
                ('count', models.IntegerField(default=0)),
                ('dataset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tags', to='gis_csdt.Dataset')),
            ],
        ),
        migrations.CreateModel(
            name='TagIndiv',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='MapPoint',
            fields=[
                ('mapelement_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='gis_csdt.MapElement')),
                ('lat', models.DecimalField(decimal_places=15, max_digits=18)),
                ('lon', models.DecimalField(decimal_places=15, max_digits=18)),
                ('field1', models.CharField(blank=True, max_length=200)),
                ('field2', models.CharField(blank=True, max_length=200)),
                ('field3', models.CharField(blank=True, max_length=200)),
                ('street', models.CharField(blank=True, max_length=200)),
                ('city', models.CharField(blank=True, max_length=100)),
                ('state', models.CharField(blank=True, max_length=2)),
                ('zipcode', models.CharField(blank=True, max_length=5)),
                ('county', models.CharField(blank=True, max_length=75)),
                ('geocoded', models.BooleanField(default=False)),
            ],
            bases=('gis_csdt.mapelement',),
        ),
        migrations.CreateModel(
            name='MapPolygon',
            fields=[
                ('mapelement_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='gis_csdt.MapElement')),
                ('lat', models.CharField(max_length=17)),
                ('lon', models.CharField(max_length=17)),
                ('field1', models.FloatField()),
                ('field2', models.FloatField()),
                ('mpoly', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
            ],
            bases=('gis_csdt.mapelement',),
        ),
        migrations.AddField(
            model_name='tagindiv',
            name='mapelement',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.MapElement'),
        ),
        migrations.AddField(
            model_name='tagindiv',
            name='tag',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.Tag'),
        ),
        migrations.AddField(
            model_name='observation',
            name='mapelement',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='observations', to='gis_csdt.MapElement'),
        ),
        migrations.AddField(
            model_name='observation',
            name='sensor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.Sensor'),
        ),
        migrations.AddField(
            model_name='mapelement',
            name='dataset',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.Dataset'),
        ),
        migrations.AddField(
            model_name='datafield',
            name='dataset',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dataFields', to='gis_csdt.Dataset'),
        ),
        migrations.AddField(
            model_name='dataelement',
            name='datafield',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dataElements', to='gis_csdt.DataField'),
        ),
        migrations.AddField(
            model_name='dataelement',
            name='denominator',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.DataElement'),
        ),
        migrations.AddField(
            model_name='dataelement',
            name='mapelement',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.MapElement'),
        ),
        migrations.AlterUniqueTogether(
            name='tagindiv',
            unique_together=set([('tag', 'mapelement')]),
        ),
        migrations.AlterUniqueTogether(
            name='observationvalue',
            unique_together=set([('observation', 'name')]),
        ),
    ]


# This file includes functions to load basic information
# should be used from the shell

import csv
import ftplib
import json
import os
import urllib
import zipfile
from django.contrib.gis.utils import LayerMapping
from models import MapPolygon, MapElement, MapPoint, GeoCoordinates, Location, DatasetNameField, Dataset, DataField, DataElement, Tag, TagIndiv
from datetime import datetime
from django.utils.timezone import utc
from django.conf import settings
from django.contrib.gis.geos import Point
from django.db import IntegrityError
from django.db.models import Max
from django.core.exceptions import ObjectDoesNotExist
from settings import DEBUG
import re
import string


def run(verbose=True, year=2010, starting_state=1):
    yn = ''
    # https://docs.djangoproject.com/en/1.7/ref/contrib/gis/layermapping/
    while DEBUG and yn != 'y':
        yn = raw_input('This process can be memory-intensive if'
                       'DEBUG = True in settings as this logs all SQL. '
                       'DEBUG is currently True. Please set this to False'
                       'if you are experiencing issues. Continue (y/n)?') \
                       .lower().strip()
        if yn == 'n':
            return
    dataset_qs = Dataset.objects.filter(name__exact=str(year)+' Census Tracts')
    if len(dataset_qs) > 0:
        ds = dataset_qs[0]
        ds.cached = datetime.utcnow().replace(tzinfo=utc),
    else:
        coor = GeoCoordinates(lat_field='INTPTLAT'+str(year)[-2:],
                              lon_field='INTPTLON'+str(year)[-2:])
        coor.save()
        names = DatasetNameField(field1_en='Land Area',
                                 field1_name='ALAND'+str(year)[-2:],
                                 field2_en='Water Area',
                                 field2_name='AWATER'+str(year)[-2:])
        names.save()
        ds = Dataset(name=str(year)+' Census Tracts',
                     cached=datetime.utcnow().replace(tzinfo=utc),
                     cache_max_age=1000,
                     name_field='NAMELSAD'+str(year)[-2:],
                     coordinates=coor,
                     names=names)
        if year == 2010:
            ds.remote_id_field = 'GEOID00'
        elif year == 2000:
            ds.remote_id_field = 'CTIDFP00'
        ds.save()

    tract_mapping = {
        'remote_id': ds.remote_id_field,
        'name': ds.name_field,
        'lat': ds.coordinates.lat_field,
        'lon': ds.coordinates.lon_field,
        'field1': ds.names.field1_name,
        'field2': ds.names.field2_name,
        'mpoly': 'MULTIPOLYGON',
    }

    ftp = ftplib.FTP('ftp2.census.gov')
    ftp.login()
    ftp.cwd("/geo/tiger/TIGER2010/TRACT/" + str(year) + "/")
    files = ftp.nlst()

    MapPolygon.objects.filter(dataset_id__isnull=True).delete()
    max_state = MapPolygon.objects.filter(dataset_id__exact=ds.id).aggregate(Max('remote_id'))
    max_state = max_state['remote_id__max']
    if max_state is not None:
        try:
            max_state = int(max_state)/1000000000
            if max_state >= starting_state:
                starting_state = max_state + 1
        except Exception:
            pass

    for i in [format(x, '#02d') for x in range(starting_state, 100)]:
        short_name = 'tl_2010_' + i + '_tract' + str(year)[-2:]
        tract_shp = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                    'data/'+short_name))
        if (not os.path.isfile(tract_shp+'.shp')
            or not os.path.isfile(tract_shp+'.shx')
            or not os.path.isfile(tract_shp+'.shp.xml')
            or not os.path.isfile(tract_shp+'.prj')
            or not os.path.isfile(tract_shp+'.dbf')):

            if short_name + '.zip' not in files:
                continue
            if verbose:
                print short_name + '.shp does not exist locally.\n\tDownloading from Census FTP...'
            try:
                # download the file
                local_file = open(tract_shp+'.zip', 'wb')
                ftp.retrbinary('RETR '+short_name+'.zip', local_file.write)
                local_file.close()
                # open the zip
                zipped = zipfile.ZipFile(tract_shp+'.zip')
                for suffix in ['.shp', '.prj', '.dbf', '.shp.xml', '.shx']:
                    zipped.extract(short_name+suffix, os.path.abspath(os.path.join(os.path.dirname(__file__), 'data')))
            except Exception as inst:
                if verbose:
                    print '\tException:', inst
                    print '\t'+short_name + '.shp did not download or unzip correctly. Moving on...'
                continue
        tract_shp = tract_shp + '.shp'
        if verbose:
            print '\tBegin layer mapping...'
        lm = LayerMapping(MapPolygon, tract_shp, tract_mapping, transform=False, encoding='iso-8859-1')

        while True:
            try:
                lm.save(strict=True, verbose=False)  # verbose)
                break
            # exception part is untested, error didn't happen again
            except Exception as inst:
                yn = ''
                while yn not in ['n', 'y']:
                    yn = raw_input('Error saving: ' + str(inst) + '\nContinue (y/n)?').strip().lower()
                if yn == 'y':
                    MapPolygon.objects.filter(dataset_id__isnull=True).filter(remote_id__startswith=i).delete()
                else:
                    break
        if verbose:
            print '\tLayer mapping done.'
        MapPolygon.objects.filter(dataset=None).update(dataset=ds)
        if verbose:
            print '\tLayer associated with dataset.'
    ftp.quit()

    if verbose:
        print 'All shapefiles added.'


'''def temp_switch():
    for poly in MapPolygon.objects.all():
        lat = poly.field1
        lon = poly.field2
        poly.field1 = poly.lat
        poly.field2 = poly.lon
        poly.lat = lat
        poly.lon = lon
        poly.save()'''


def recount():
    all_tags = Tag.objects.all()
    for tag in all_tags:
        tag.recount(save=True)


def get_income(year=2010, ds_id=0, to_process=None):
    try:
        # check the format
        if to_process is None:
            raise
        for v in to_process:
            if 'variable' not in v or 'variable_type' not in v:
                raise
    except Exception:
        # http://www.census.gov/data/developers/data-sets/acs-survey-5-year-data.html
        # http://api.census.gov/data/2010/acs5/variables.html
        #              median household income                                     Total population
        to_process = [{'variable': 'B19013_001E', 'variable_type': DataField.INTEGER},
                      {'variable': 'B01003_001E', 'variable_type': DataField.INTEGER}]
        '''to_process = [{'variable':'B02001_001E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_002E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_003E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_004E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_005E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_006E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_007E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_008E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_009E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B02001_010E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B17001_001E', 'variable_type':DataField.INTEGER},
                                    {'variable':'B17001_002E', 'variable_type':DataField.INTEGER}
                                    ]'''
    key = settings.CENSUS_API_KEY

    ds = Dataset.objects.filter(name__icontains='census').filter(name__contains=str(year)).all()
    if len(ds) == 0:
        print 'No Census dataset exists. Aborting'
        return
    elif len(ds) > 1:
        if ds_id > 0:
            ds = [ds.get(pk=ds_id)]
        else:
            print 'More than one 2010 Census dataset exists. Using ID %d' % (ds[0].id)

    dfs = []
    get = ''
    for item in to_process:
        try:
            dfs.append(DataField.objects.filter(dataset_id__exact=ds[0].id).get(field_name=item['variable']))
        except Exception:
            request = 'http://api.census.gov/data/2010/acs5/variables/%s.json' % (item['variable'])
            data = json.loads(urllib.urlopen(request).read())
            dfs.append(DataField(dataset=ds[0], field_en=data['label'],
                       field_longname=data['concept'][7:].strip(),
                       field_name=item['variable'],
                       field_type=item['variable_type']))
            if len(dfs[-1].field_en) > DataField._meta.get_field('field_en').max_length:
                dfs[-1].field_en = dfs[-1].field_en[:DataField._meta.get_field('field_en').max_length]
            if len(dfs[-1].field_longname) > DataField._meta.get_field('field_longname').max_length:
                dfs[-1].field_longname = dfs[-1].field_longname[:DataField._meta.get_field('field_longname').max_length]
            dfs[-1].save()
        get = get + ',' + item['variable']
    get = get.strip(', ')

    counties = json.loads(urllib.urlopen('http://api.census.gov/data/2010/acs5?key='+key+'&get=NAME&for=county:*').read())
    # i could technically call this once, but it's just too much data at once so i'm splitting by state
    for county in counties[1:]:
        # get the data
        request = 'http://api.census.gov/data/2010/acs5?key=%s&get=%s,NAME&for=tract:*&in=state:%s,county:%s' % (key, get, county[1], county[2])
        try:
            data = json.loads(urllib.urlopen(request).read())
        except Exception:
            continue
        converted = {}
        # locations of basic data in each list
        # format like:
        # [["B19013_001E","B01003_001E","NAME","state","county","tract"],
        # ["32333","2308","Census Tract 1, Albany County, New York","36","001","000100"],
        # ...]
        for col in range(len(data[0])):
            if data[0][col] == 'NAME':
                n = col
            elif data[0][col] == 'state':
                s = col
            elif data[0][col] == 'county':
                c = col
            elif data[0][col] == 'tract':
                t = col
        for d in data[1:]:
            census_tract = d[s] + d[c] + d[t]
            converted[census_tract] = {}
            for num in range(n):
                if d[num] is not None:
                    converted[census_tract][data[0][num]] = d[num].strip()
        # converted now looks like
        # {"36001000100": {"B19013_001E": "32333","B01003_001E": "2308"},
        #  "36001000200": {"B19013_001E": "25354","B01003_001E": "5506"},...}
        for poly in MapPolygon.objects.filter(dataset=ds[0]).filter(remote_id__startswith=county[1]+county[2]):
            if poly.remote_id in converted:
                recursive_link = {}
                for df in dfs:
                    if df.field_name in converted[poly.remote_id]:
                        de = DataElement(datafield=df, mapelement=poly)
                        if df.field_type == DataField.INTEGER:
                            try:
                                de.int_data = int(converted[poly.remote_id][df.field_name])
                            except Exception:
                                print 'integer conversion failed for census tract %s, field %s' % (poly.remote_id, df.field_name)
                                print 'value:', converted[poly.remote_id][df.field_name]
                        elif df.field_type == DataField.FLOAT:
                            try:
                                de.float_data = float(converted[poly.remote_id][df.field_name])
                            except Exception:
                                print 'float conversion failed for census tract %s, field %s' % (poly.remote_id, df.field_name)
                        elif df.field_type == DataField.STRING:
                            if len(converted[poly.remote_id][df.field_name]) > 200:
                                print 'string overload - string truncated as shown:'
                                print '%s[%s]' % (converted[poly.remote_id][df.field_name][:200], 
                                                  converted[poly.remote_id][df.field_name][200:])
                                de.char_data = converted[poly.remote_id][df.field_name][:200]
                            else:
                                de.char_data = converted[poly.remote_id][df.field_name]
                        if df.field_name[-4:] == '001E':
                            de.save()
                            recursive_link[df.field_name[:-4]] = de
                        try:
                            de.denominator = recursive_link[df.field_name[:-4]]
                        except Exception:
                            pass
                        de.save()

            else:
                print 'ERROR: Tract #%s is not in the dataset' % (poly.remote_id)


def add_denominator():
    for df_denom in DataField.objects.filter(field_name__contains='_001E'):
        for df in DataField.objects.filter(field_name__contains=df_denom.field_name[:-4]).filter(dataset_id__exact=df_denom.dataset_id):
            for de in DataElement.objects.filter(datafield_id__exact=df.id).filter(denominator__isnull=True):
                try:
                    de.denominator = DataElement.objects.filter(datafield_id__exact=df_denom.id).get(mapelement_id=de.id)
                    de.save()
                except Exception:
                    continue


def del_all():
    DataField.objects.all().delete()


def tag_by_keyword(keyword='pizza', dataset=2, tag='pizza'):
    tag_helper([{'keyword': keyword}], 'keyword', dataset, tag)


def tag_by_name(filename='fastfood.json', name_field='Company', dataset=2, tag='fast food'):
    data = json.loads(open(os.path.abspath(os.path.join(os.path.dirname(__file__), 'data/'+filename))).read())
    data = data['data']
    tag_helper(data, name_field, dataset, tag)


def tag_helper(data, name_field, dataset, tag):
    mps = MapPoint.objects.filter(dataset_id=dataset)
    tags = Tag.objects.filter(dataset_id=dataset).filter(tag=tag)
    if tags.count() > 1:
        appr_tags = tags.filter(approved=True)
        if appr_tags.count() > 0:
            tags = appr_tags
        tag = tags[0]
    elif tags.count() == 0:
        tag = Tag(dataset_id=dataset, tag=tag, approved=True)
        tag.save()
    else:
        tag = tags[0]

    regex = re.compile('[%s]' % re.escape(string.punctuation))
    for item in data:
        name = regex.sub(r'[\'\'\.-]*', item[name_field])
        print name
        for mp in mps.filter(name__iregex=name):
            print mp.name
            t = TagIndiv(tag=tag, mapelement=mp)
            try:
                t.save()
            except IntegrityError:
                # violate unique restraint so just don't save it
                pass


def add_point_to_mp(dataset=None):
    mps = MapPoint.objects.filter(point__isnull=True).exclude(lat__isnull=True).exclude(lon__isnull=True)
    if dataset is not None:
        mps = mps.filter(dataset_id__exact=dataset)
    for mp in mps:
        try:
            mp.point = Point(float(mp.lon), float(mp.lat))
            mp.save()
        except Exception:
            pass


def hazardous_waste(year=2011, verbose=True):
    try:
        dataset = Dataset.objects.get(name="Hazardous Waste Sites "+str(year))
        dataset.cached = datetime.utcnow().replace(tzinfo=utc)
    except ObjectDoesNotExist:
        coor = GeoCoordinates(lat_field="Latitude",
                              lon_field="Longitude")
        coor.save()
        names = DatasetNameField(field1_en="Generator Status",
                                 field1_name="Generator Status",
                                 field2_en="Biennial Report Link",
                                 field2_name="Biennial Report Link")
        names.save()
        location = Location(street_field="Address",
                            city_field="City",
                            state_field="State",
                            zipcode_field="ZIP Code",
                            county_field="County")
        dataset = Dataset(
            name="Hazardous Waste Sites "+str(year),
            url='/data/ej/'+str(year)+'/',
            cached=datetime.utcnow().replace(tzinfo=utc),
            cache_max_age=1000,
            remote_id_field="Handler ID",
            name_field="Handler Name",
            location=location,
            coordinates=coor,
            names=names
            needs_geocoding=False)
    dataset.save()

    MapPoint.objects.filter(dataset=dataset).delete()

    for state in ['AL', 'AK', 'AZ', 'AR', 'CA', 'CO', 'CT', 'DE',
                  'DC', 'FL', 'GA', 'HI', 'ID', 'IL', 'IN', 'IA',
                  'KS', 'KY', 'LA', 'ME', 'MD', 'MA', 'MI', 'MN',
                  'MS', 'MO', 'MT', 'NE', 'NV', 'NH', 'NJ', 'NM',
                  'NY', 'NC', 'ND', 'OH', 'OK', 'OR', 'PA', 'RI',
                  'SC', 'SD', 'TN', 'TX', 'UT', 'VT', 'VA', 'WA',
                  'WV', 'WI', 'WY']:
        short_name = 'Envirofacts_Biennial_Report_Search ' + state + '.CSV'
        path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'data/ej/'+str(year)+'/'+short_name))
        if not os.path.isfile(path):
            if verbose:
                print 'No file %s exists.' % (short_name)
            short_name = str(year)+' '+state+'.CSV'
            path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'data/ej/'+str(year)+'/'+short_name))
            if not os.path.isfile(path):
                if verbose:
                    print 'No file %s exists.' % (short_name)
                continue
        if verbose:
            print 'Opening file %s' % (short_name)
        readfile = csv.reader(open(path, 'rb'))
        # verify
        row = readfile.next()
        locs = {}
        for i in range(len(row)):
            if row[i] == dataset.remote_id_field:
                locs['remote_id'] = i
            elif row[i] == dataset.name_field:
                locs['name'] = i
            elif row[i] == dataset.location.street_field:
                locs['street'] = i
            elif row[i] == dataset.location.city_field:
                locs['city'] = i
            elif row[i] == dataset.location.state_field:
                locs['state'] = i
            elif row[i] == dataset.location.zipcode_field:
                locs['zipcode'] = i
            elif row[i] == dataset.location.county_field:
                locs['county'] = i
            elif row[i] == dataset.coordinates.lat_field:
                locs['lat'] = i
            elif row[i] == dataset.coordinates.lon_field:
                locs['lon'] = i
            elif row[i] == dataset.names.field1_name:
                locs['field1'] = i
            elif row[i] == dataset.names.field2_name:
                locs['field2'] = i
        for row in readfile:
            kwargs = {'dataset': dataset}
            for key in locs:
                if key in ['lat', 'lon']:
                    try:
                        kwargs[key] = float(row[locs[key]])
                    except Exception:
                        kwargs[key] = 0.
                elif MapPoint._meta.get_field(key).max_length < len(row[locs[key]]):
                    kwargs[key] = row[locs[key]][:MapPoint._meta.get_field(key).max_length]
                else:
                    kwargs[key] = row[locs[key]]
            try:
                kwargs['point'] = Point(kwargs['lon'], kwargs['lat'])
            except Exception:
                if verbose:
                    print '\tInvalid lat/long for row: %s' % (row)
                    print '\tLat: %f Lon: %f' % (kwargs['lat'], kwargs['lon'])
                continue
            mp = MapPoint(**kwargs)
            mp.save()
        if verbose:
            print 'File "%s" done processing' % (short_name)


def load_from_se(year=2000):
    flds = {'SE_T001_001': 'T1. Total Population',
            # 'SE_T014_001':'Total Population',
            # 'SE_T014_002':'White Alone',
            # 'SE_T014_003':'Black or African American Alone',
            # 'SE_T014_004':'American Indian and Alaska Native Alone',
            # 'SE_T014_005':'Asian Alone',
            # 'SE_T014_006':'Native Hawaiian and Other Pacific Islander Alone',
            # 'SE_T014_007':'Some other race Alone',
            # 'SE_T014_008':'Two or more races',
            # 'SE_T015_001':'Total Population',
            # 'SE_T015_002':'Not Hispanic or Latino',
            # 'SE_T015_003':'Not Hispanic or Latino: White Alone',
            # 'SE_T015_004':'Not Hispanic or Latino: Black or African American Alone',
            # 'SE_T015_005':'Not Hispanic or Latino: American Indian and Alaska Native Alone',
            # 'SE_T015_006':'Not Hispanic or Latino: Asian Alone',
            # 'SE_T015_007':'Not Hispanic or Latino: Native Hawaiian and Other Pacific Islander Alone',
            # 'SE_T015_008':'Not Hispanic or Latino: Some other race Alone',
            # 'SE_T015_009':'Not Hispanic or Latino: Two or more races',
            # 'SE_T015_010':'Hispanic or Latino',
            # 'SE_T015_011':'Hispanic or Latino: White Alone',
            # 'SE_T015_012':'Hispanic or Latino: Black or African American Alone',
            # 'SE_T015_013':'Hispanic or Latino: American Indian and Alaska Native Alone',
            # 'SE_T015_014':'Hispanic or Latino: Asian Alone',
            # 'SE_T015_015':'Hispanic or Latino: Native Hawaiian and Other Pacific Islander Alone',
            # 'SE_T015_016':'Hispanic or Latino: Some other race Alone',
            # 'SE_T015_017':'Hispanic or Latino: Two or more races',
            'SE_T016_001': 'T16. Total Population',
            'SE_T016_002': 'T16. White Alone',
            'SE_T016_003': 'T16. Black or African American Alone',
            'SE_T016_004': 'T16. American Indian and Alaska Native Alone',
            'SE_T016_005': 'T16. Asian Alone',
            'SE_T016_006': 'T16. Native Hawaiian and Other Pacific Islander Alone',
            'SE_T016_007': 'T16. Some other race Alone',
            'SE_T016_008': 'T16. Two or more races',
            'SE_T017_001': 'T17. Total Population',
            'SE_T017_002': 'T17. Not Hispanic or Latino',
            # 'SE_T017_003': 'Not Hispanic or Latino: White Alone',
            # 'SE_T017_004': 'Not Hispanic or Latino: Black or African American Alone',
            # 'SE_T017_005': 'Not Hispanic or Latino: American Indian and Alaska Native Alone',
            # 'SE_T017_006': 'Not Hispanic or Latino: Asian Alone',
            # 'SE_T017_007': 'Not Hispanic or Latino: Native Hawaiian and Other Pacific Islander Alone',
            # 'SE_T017_008': 'Not Hispanic or Latino: Some other race Alone',
            # 'SE_T017_009': 'Not Hispanic or Latino: Two or more races',
            'SE_T017_010': 'T17. Hispanic or Latino',
            # 'SE_T017_011': 'Hispanic or Latino: White Alone',
            # 'SE_T017_012': 'Hispanic or Latino: Black or African American Alone',
            # 'SE_T017_013': 'Hispanic or Latino: American Indian and Alaska Native Alone',
            # 'SE_T017_014': 'Hispanic or Latino: Asian Alone',
            # 'SE_T017_015': 'Hispanic or Latino: Native Hawaiian and Other Pacific Islander Alone',
            # 'SE_T017_016': 'Hispanic or Latino: Some other race Alone',
            # 'SE_T017_017': 'Hispanic or Latino: Two or more races',
            # 'SE_T092_001': 'Households',
            # 'SE_T092_002': 'Household Income: Less than $10,000',
            # 'SE_T092_003': 'Household Income: $10,000 to $14,999',
            # 'SE_T092_004': 'Household Income: $15,000 to $19,999',
            # 'SE_T092_005': 'Household Income: $20,000 to $24,999',
            # 'SE_T092_006': 'Household Income: $25,000 to $29,999',
            # 'SE_T092_007': 'Household Income: $30,000 to $34,999',
            # 'SE_T092_008': 'Household Income: $35,000 to $39,999',
            # 'SE_T092_009': 'Household Income: $40,000 to $44,999',
            # 'SE_T092_010': 'Household Income: $45,000 to $49,999',
            # 'SE_T092_011': 'Household Income: $50,000 to $59,999',
            # 'SE_T092_012': 'Household Income: $60,000 to $74,999',
            # 'SE_T092_013': 'Household Income: $75,000 to $99,999',
            # 'SE_T092_014': 'Household Income: $100,000 to $124,999',
            # 'SE_T092_015': 'Household Income: $125,000 to $149,999',
            # 'SE_T092_016': 'Household Income: $150,000 to $199,999',
            # 'SE_T092_017': 'Household Income: $200,000 or more',
            # 'SE_T092A001': 'Households',
            # 'SE_T092A002': 'Less than $10,000',
            # 'SE_T092A003': 'Less than $15,000',
            # 'SE_T092A004': 'Less than $20,000',
            # 'SE_T092A005': 'Less than $25,000',
            # 'SE_T092A006': 'Less than $30,000',
            # 'SE_T092A007': 'Less than $35,000',
            # 'SE_T092A008': 'Less than $40,000',
            # 'SE_T092A009': 'Less than $45,000',
            # 'SE_T092A010': 'Less than $50,000',
            # 'SE_T092A011': 'Less than $60,000',
            # 'SE_T092A012': 'Less than $75,000',
            # 'SE_T092A013': 'Less than $100,000',
            # 'SE_T092A014': 'Less than $125,000',
            # 'SE_T092A015': 'Less than $150,000',
            # 'SE_T092A016': 'Less than $200,000',
            # 'SE_T092B001': 'Households',
            # 'SE_T092B002': 'More than $10,000',
            # 'SE_T092B003': 'More than $15,000',
            # 'SE_T092B004': 'More than $20,000',
            # 'SE_T092B005': 'More than $25,000',
            # 'SE_T092B006': 'More than $30,000',
            # 'SE_T092B007': 'More than $35,000',
            # 'SE_T092B008': 'More than $40,000',
            # 'SE_T092B009': 'More than $45,000',
            # 'SE_T092B010': 'More than $50,000',
            # 'SE_T092B011': 'More than $60,000',
            # 'SE_T092B012': 'More than $75,000',
            # 'SE_T092B013': 'More than $100,000',
            # 'SE_T092B014': 'More than $125,000',
            # 'SE_T092B015': 'More than $150,000',
            # 'SE_T092B016': 'More than $200,000',
            # 'SE_T093_001': 'Median household income In 1999 Dollars',
            # 'SE_T096_001': 'Average household income In 1999 Dollars',
            # 'SE_T162_001': 'Owner-occupied housing units',
            # 'SE_T162_002': 'Value For All Owner-occupied housing units: Less than $20,000',
            # 'SE_T162_003': 'Value For All Owner-occupied housing units: $20,000 to $49,999',
            # 'SE_T162_004': 'Value For All Owner-occupied housing units: $50,000 to $99,999',
            # 'SE_T162_005': 'Value For All Owner-occupied housing units: $100,000 to $149,999',
            # 'SE_T162_006': 'Value For All Owner-occupied housing units: $150,000 to $299,999',
            # 'SE_T162_007': 'Value For All Owner-occupied housing units: $300,000 to $499,999',
            # 'SE_T162_008': 'Value For All Owner-occupied housing units: $500,000 to $749,999',
            # 'SE_T162_009': 'Value For All Owner-occupied housing units: $750,000 to $999,999',
            # 'SE_T162_010': 'Value For All Owner-occupied housing units: $1,000,000 or more',
            # 'SE_T184_001': 'T184. Population for whom poverty status is determined:',
            # 'SE_T184_002': 'T184. Under .50',
            # 'SE_T184_003': 'T184. .50 to .74',
            # 'SE_T184_004': 'T184. .75 to .99',
            # 'SE_T184_005': 'T184. 1.00 to 1.49',
            # 'SE_T184_006': 'T184. 1.50 to 1.99',
            # 'SE_T184_007': 'T184. 2.00 and over'}
            }

    try:
        dataset = Dataset.objects.get(name=str(year)+' Census Tracts')
        # temp = DataElement.objects.filter(datafield__dataset_id=dataset.id).aggregate(max=Max('id'),min=Min('id'))
        # for i in range(temp['min']/10000*10000+10000,temp['max']/10000*10000+10000,10000):
        #     DataElement.objects.filter(datafield__dataset_id=dataset.id,id__lte=i).delete()
        # print 'dataelements deleted'
    except Exception as e:
        print e
        print 'Dataset "'+str(year), 'Census Tracts" does not exist'
        return
    for key in flds:
        try:
            df = DataField.objects.filter(dataset_id=dataset.id).get(field_name=key)
        except Exception:
            df = DataField(dataset=dataset, field_name=key,
                           field_en=flds[key], field_longname=flds[key],
                           field_type=DataField.INTEGER)
            df.save()
        flds[key] = df
    for i in range(1, 6):
        filename = 'part' + str(i) + ' tracts.csv'
        filename = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                   'data/' + str(year) + '/' + filename))
        openfile = open(filename, 'rb')
        csv_file = csv.DictReader(openfile)
        for line in csv_file:
            geo = '0'*(11-len(line['Geo_FIPS'])) + line['Geo_FIPS']
            try:
                me = MapElement.objects.filter(dataset_id=dataset.id).get(remote_id=geo)
            except Exception:
                print 'GEOID:', line['Geo_FIPS'], 'does not exist'
                continue
            current = {}
            for key in line:
                if key in flds:
                    try:
                        de = DataElement(datafield=flds[key], mapelement=me, int_data=int(line[key]))
                    except Exception:
                        continue
                    de.save()
                    if key[-4:] == '_001':
                        current[key[:-4]] = de
                    try:
                        de.denominator = current[key[:-4]]
                        de.save()
                    except Exception:
                        pass
        print 'file', filename, 'processed'



from django.conf.urls import include, url
from rest_framework import routers
from gis_csdt import views, templates
from django.contrib import admin
admin.autodiscover()

router = routers.DefaultRouter()
router.register(r'api-ds', views.DatasetViewSet)
router.register(r'api-location', views.LocationViewSet, base_name='location')
router.register(r'api-geocoor', views.GeoCoordinatesViewSet,
                base_name='geocoor')
router.register(r'rpi-dsnames', views.DatasetNameFieldViewSet,
                base_name='ds_names')
router.register(r'api-mp', views.MapPointViewSet, base_name='mappoint')
router.register(r'api-newtag', views.NewTagViewSet)
router.register(r'api-poly', views.MapPolygonViewSet, base_name='mappolygon')
router.register(r'api-tag', views.NewTagViewSet)
router.register(r'api-test', views.TestView, base_name='mp')
router.register(r'api-count', views.CountPointsInPolygonView,
                base_name='count_points')
# router.register(r'api-tag/count', views.TagCountViewSet, base_name = 'tag')
router.register(r'api-dist', views.AnalyzeAreaAroundPointView,
                base_name='area')
router.register(r'api-dist2', views.AnalyzeAreaAroundPointNoValuesView,
                base_name='area2')
router.register(r'api-newsensor', views.NewSensorView)
router.register(r'api-datapoint', views.SubmitDataPointView)

urlpatterns = [
    url(r'^', include(router.urls)),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    url(r'api-SMS', views.SMSSubmitDataPointView, name='SMS-data'),
    url(r'api-addmp', views.AddMapPointView.as_view(), name='Add_MP'),
    url(r'api-addds', views.AddDatasetView.as_view(), name='Add_DS'),
    # url(r'^api-count/', views.CountPointsInPolygonView.as_view(), name='count'),
    # url(r'^api-dist/', views.AnalyzeAreaAroundPointView, name='distance'),
    url(r'^around-point/(?P<mappoint_id>[0-9]+)/$', templates.AroundPointView),
    url(r'^around-point/$', templates.AroundPointView),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-06 20:29
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('gis_csdt', '0002_phonenumber'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataPoint',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.DecimalField(decimal_places=15, max_digits=30)),
                ('point', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='points', to='gis_csdt.MapPoint')),
            ],
        ),
        migrations.RemoveField(
            model_name='sensor',
            name='sensor_type',
        ),
        migrations.AddField(
            model_name='sensor',
            name='accuracy',
            field=models.CharField(default=b'accuracy', max_length=100),
        ),
        migrations.AddField(
            model_name='sensor',
            name='metric',
            field=models.CharField(default=b'metric', max_length=100),
        ),
        migrations.AddField(
            model_name='sensor',
            name='model_number',
            field=models.CharField(default=b'model_number', max_length=100),
        ),
        migrations.AddField(
            model_name='sensor',
            name='supplier',
            field=models.CharField(default=b'supplier', max_length=100),
        ),
        migrations.AlterField(
            model_name='sensor',
            name='name',
            field=models.CharField(default=b'name', max_length=100, unique=True),
        ),
        migrations.AddField(
            model_name='datapoint',
            name='sensor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sensors', to='gis_csdt.Sensor'),
        ),
        migrations.AddField(
            model_name='datapoint',
            name='user',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



from django_extensions.management.jobs import DailyJob
from gis_csdt.models import Dataset
from gis_csdt.load import add_point_to_mp


class Job(DailyJob):
    help = 'Daily dataset updater'

    def execute(self):
        datasets = Dataset.objects.all()
        for dataset in datasets:
            print 'Update dataset:', dataset
            try:
                dataset.update_mappoints()
            except Exception as inst:
                print type(inst)     # the exception instance
                print inst.args      # arguments stored in .args
                print inst           # __str__ allows args to be printed directly
        # not sure why I still have to do this but i do
        add_point_to_mp()



from django.contrib.gis.geos import Polygon
from django.contrib.gis.measure import Distance
from math import pow, sqrt, asin, sin, cos, radians
from geopy import Point as gpPoint
from geopy.distance import VincentyDistance

EARTH_RADIUS = Distance(km=6371)


def haversin(theta):
    return pow(sin(theta/2), 2)


def ahaversin(haversine):
    return 2 * asin(sqrt(haversine))


def haversine_distance(lat1, lon1, lat2, lon2):
    # convert to radians
    lat1 = radians(lat1)
    lon1 = radians(lon1)
    lat2 = radians(lat2)
    lon2 = radians(lon2)
    diff_lat = lat2-lat1
    diff_lon = lon2-lon1
    haversine = haversin(diff_lat) + cos(lat1) * cos(lat2) * haversin(diff_lon)
    assert haversine >= 0 and haversine <= 1
    # get distance from haversine
    return EARTH_RADIUS * ahaversin(haversine)


def circle_as_polygon(lat, lon, n=12, distance=Distance(km=1)):
    # this can be done all in degrees because that's what geopy uses!
    points = []
    # build points, loop around circle
    for angle in [360.*i/n for i in range(0, n)]:
        curr = VincentyDistance(kilometers=distance.km) \
               .destination(gpPoint(lat, lon), angle)
        points.append((curr.longitude, curr.latitude))
    # loop back to the first point for the n+1th point in the list
    points.append(points[0])
    return Polygon(tuple(points))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-07-02 17:13
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def add_datapoints_to_sensor(apps, schema_editor):
    DataPoint = apps.get_model('gis_csdt', 'DataPoint')
    Sensor = apps.get_model('gis_csdt', 'Sensor')
    for dp in DataPoint.objects.all():
        sensor = Sensor.objects.get(pk=dp.sensor.id)
        if sensor.mappoint == None:
            sensor.mappoint = dp.point
        if sensor.user == None:
            sensor.user = dp.user
        sensor.datapoints.add(dp)
        sensor.save()


def update_dataset(apps, schema_editor):
    Dataset = apps.get_model('gis_csdt', 'Dataset')
    Location = apps.get_model('gis_csdt', 'Location')
    GeoCoordinates = apps.get_model('gis_csdt', 'GeoCoordinates')
    DatasetNameField = apps.get_model('gis_csdt', 'DatasetNameField')
    for ds in Dataset.objects.all():
        coor_attrs = ['lat_field', 'lon_field']
        location_attrs = ['street_field', 'city_field', 'state_field',
                          'county_field','zipcode_field']
        names_attrs = ['field1_en', 'field1_name', 'field2_en',
                       'field2_name', 'field3_en', 'field3_name']
        #if (not None) in [getattr(ds, attr) for attr in coor_attrs]:
        coor = GeoCoordinates()
        coor.save()
        for attr in coor_attrs:
            setattr(coor, attr, getattr(ds, attr))
            coor.save()
        ds.coordinates = coor
        ds.save()
        #if (not None) in [getattr(ds, attr) for attr in location_attrs]:
        location = Location()
        location.save()
        for attr in location_attrs:
            setattr(location, attr, getattr(ds, attr))
            location.save()
        ds.location = location
        ds.save()
        #if (not None) in [getattr(ds, attr) for attr in names_attrs]:
        names = DatasetNameField()
        names.save()
        for attr in names_attrs:
            setattr(names, attr, getattr(ds, attr))
            names.save()
        ds.names = names
        ds.save()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('gis_csdt', '0003_auto_20180806_1629'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='observation',
            name='mapelement',
        ),
        migrations.RemoveField(
            model_name='observation',
            name='sensor',
        ),
        migrations.AlterUniqueTogether(
            name='observationvalue',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='observationvalue',
            name='observation',
        ),
        migrations.CreateModel(
            name='DatasetNameField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field1_en', models.CharField(blank=True, max_length=150)),
                ('field1_name', models.CharField(blank=True, max_length=50)),
                ('field2_en', models.CharField(blank=True, max_length=150)),
                ('field2_name', models.CharField(blank=True, max_length=50)),
                ('field3_en', models.CharField(blank=True, max_length=150)),
                ('field3_name', models.CharField(blank=True, max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='GeoCoordinates',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lat_field', models.CharField(blank=True, default=b'latitude', max_length=50)),
                ('lon_field', models.CharField(blank=True, default=b'longitude', max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('street_field', models.CharField(blank=True, default=b'street', max_length=50)),
                ('city_field', models.CharField(blank=True, default=b'city', max_length=50)),
                ('state_field', models.CharField(blank=True, default=b'state', max_length=50)),
                ('zipcode_field', models.CharField(blank=True, default=b'zip', max_length=50)),
                ('county_field', models.CharField(blank=True, default=b'county', max_length=50)),
            ],
        ),
        migrations.AddField(
            model_name='datapoint',
            name='time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sensor',
            name='datapoints',
            field=models.ManyToManyField(blank=True, to='gis_csdt.DataPoint'),
        ),
        migrations.AddField(
            model_name='sensor',
            name='mappoint',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='points', to='gis_csdt.MapPoint'),
        ),
        migrations.AddField(
            model_name='sensor',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.RunPython(add_datapoints_to_sensor),
        migrations.AddField(
            model_name='dataset',
            name='coordinates',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.GeoCoordinates'),
        ),
        migrations.AddField(
            model_name='dataset',
            name='location',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.Location'),
        ),
        migrations.AddField(
            model_name='dataset',
            name='names',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='gis_csdt.DatasetNameField'),
        ),
        migrations.RunPython(update_dataset),
        migrations.RemoveField(
            model_name='datapoint',
            name='point',
        ),
        migrations.RemoveField(
            model_name='datapoint',
            name='sensor',
        ),
        migrations.RemoveField(
            model_name='datapoint',
            name='user',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='city_field',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='county_field',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='field1_en',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='field1_name',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='field2_en',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='field2_name',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='field3_en',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='field3_name',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='lat_field',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='lon_field',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='state_field',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='street_field',
        ),
        migrations.RemoveField(
            model_name='dataset',
            name='zipcode_field',
        ),
        migrations.DeleteModel(
            name='Observation',
        ),
        migrations.DeleteModel(
            name='ObservationValue',
        ),
    ]



from django.contrib import admin
from gis_csdt.models import Location, GeoCoordinates, DatasetNameField, \
                            Dataset, MapPoint, PhoneNumber, Tag, MapPolygon, \
                            TagIndiv, DataField, DataElement, Sensor, DataPoint


class DatasetAdmin(admin.ModelAdmin):
    fieldsets = [
        (None,              {'fields': ['name', 'url', 'cache_max_age']}),
        ('Field Mapping',   {'fields': ['remote_id_field', 'name_field',
                                        'location', 'coordinates']}),
        ('Custom Field',    {'fields': ['names']}),
        ('Geocoding',       {'fields': ['needs_geocoding']})]


class SensorAdmin(admin.ModelAdmin):
    filter_horizontal = ('datapoints', )


admin.site.register(Dataset, DatasetAdmin)
# for testing
admin.site.register(MapPoint)
admin.site.register(Tag)
admin.site.register(MapPolygon)
admin.site.register(TagIndiv)
admin.site.register(DataField)
admin.site.register(DataElement)
admin.site.register(Sensor, SensorAdmin)
admin.site.register(DataPoint)
admin.site.register(PhoneNumber)
admin.site.register(Location)
admin.site.register(GeoCoordinates)
admin.site.register(DatasetNameField)



from django.dispatch import receiver
from django.db.models.signals import post_save
from gis_csdt.models import Tag, TagIndiv


# @receiver(post_save, sender=Dataset)
def update_mappoints(sender, **kwargs):
    dataset = kwargs.get('instance')
    dataset.update_mappoints()
    # post_save.connect(update_mappoints, sender=Dataset)


@receiver(post_save, sender=TagIndiv)
def update_tag_count(sender, **kwargs):
    if kwargs.get('created'):  # if this is a new TagIndiv
        # increment the count and save
        kwargs.get('instance').tag.increment_count(save=True)


@receiver(post_save, sender=Tag)
def clean_tags(sender, **kwargs):
    # all tags must be lowercase without leading or trailing whitespace
    kwargs.get('instance').tag = kwargs.get('instance').tag.strip().lower()



from rest_framework import status
from rest_framework.test import APITestCase, APIClient
from django.test import TestCase
from gis_csdt.models import DataPoint, Dataset, Location, MapPoint, PhoneNumber, Sensor
from django.contrib.auth.models import User
from gis_csdt.views import DataToGSM7
from django.contrib.auth import get_user_model
from django.test import LiveServerTestCase
import urllib

User = get_user_model()


class AllViewTestsNoData(APITestCase):
    def test_no_datasets(self):
        response = self.client.get('/api-ds/')
        self.assertEqual(response.status_code, 200)

    def test_no_mappoints(self):
        response = self.client.get('/api-mp/')
        self.assertEqual(response.status_code, 200)

    def test_no_newtags(self):
        response = self.client.get('/api-newtag/')
        self.assertEqual(response.status_code, 200)

    def test_no_polygons(self):
        response = self.client.get('/api-poly/')
        self.assertEqual(response.status_code, 200)

    def test_no_mappoints_geojson(self):
        response = self.client.get('/api-test/')
        self.assertEqual(response.status_code, 200)

    def test_no_mappolygons_count_of_points(self):
        response = self.client.get('/api-count/')
        self.assertEqual(response.status_code, 200)

    def test_no_mappolygons_analysis_around_point(self):
        response = self.client.get('/api-dist/')
        self.assertEqual(response.status_code, 400)


class SMSCreateData(LiveServerTestCase):
    def setUp(self):
        self.client = APIClient()
        self.user = User.objects.create_superuser(username='test',
                                                  email='test@test.test',
                                                  password='test')
        self.assertTrue(self.client.login(username='test', password='test'))

    def test_SMS(self):
        set = Dataset.objects.create(name='test')
        set.save()
        mappoint = MapPoint.objects.create(lat=0, lon=0)
        mappoint.save()
        sensor = Sensor.objects.create(name='test', mappoint=mappoint, user=self.user)
        sensor.save()
        phNum = PhoneNumber.objects.create(phone_number=11111111111, user=self.user)
        phNum.save()
        data = [1, 0, 128, 129, 300, 10001]
        data[0] = sensor.id
        string = DataToGSM7(data)
        postData = {'Body': string.encode('utf-8'), 'From': phNum.phone_number}
        response = self.client.post('/api-SMS/', urllib.urlencode(postData), content_type='application/x-www-form-urlencoded')
        self.assertEqual(DataPoint.objects.get(pk=6).value, 10001)


class TestAddMapPointAPI(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.user = User.objects.create_superuser(username='test',
                                                  email='test@test.test',
                                                  password='test')
        self.assertTrue(self.client.login(username='test', password='test'))

    def test_api_can_add_mappoint(self):
        original_count = MapPoint.objects.all().count()
        self.mp_data = {'lat': 31.7, 'lon': 68.9}
        self.response = self.client.post('/api-addmp/', self.mp_data, format="json")
        self.assertEqual(self.response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(MapPoint.objects.all().count(), original_count + 1)


class TestAddDatasetAPI(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.user = User.objects.create_superuser(username='test',
                                                  email='test@test.test',
                                                  password='test')
        self.assertTrue(self.client.login(username='test', password='test'))

    def test_api_can_add_dataset(self):
        original_count = Dataset.objects.all().count()
        location = Location.objects.create(state_field='NY')
        location.save()
        self.ds_data = {'name': 'Catskill', 'location_id': location.id}
        self.response = self.client.post('/api-addds/', self.ds_data, format="json")
        self.assertEqual(self.response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Dataset.objects.all().count(), original_count + 1)
        self.assertEqual(Dataset.objects.get(name='Catskill').location.state_field, 'NY')


class TestAroundPointView(TestCase):

    def test_can_load_page(self):
        response = self.client.get('/around-point/')
        self.assertEqual(response.status_code, 200)

    def test_contains_mappoint(self):
        mp1 = MapPoint(lat=43.0831, lon=73.7846)
        mp1.save()
        mp2 = MapPoint(lat=20.1515, lon=25.2523)
        mp2.save()
        response = self.client.get('/api-dist/', {'min_lat': 12.12, 'max_lat': 108.00, 'dataset': '1,2', 'unit': 'km'})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, str(mp1))
        response = self.client.get('/around-point/%d/' % mp2.id)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, str(mp2))


class TestAroundPointNoValueView(TestCase):

    def test_can_load_page(self):
        self.user = User.objects.create_superuser(username='test',
                                                  email='test@test.test',
                                                  password='test')
        self.assertTrue(self.client.login(username='test', password='test'))
        mp1 = MapPoint(lat=43.0831, lon=73.7846)
        mp1.save()
        response = self.client.get('/api-dist2/', {'min_lat': 12.12, 'max_lat': 108.00, 'dataset': '1,2', 'unit': 'km'})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, str(mp1))
        response = self.client.get('/api-dist2/', {'min_lat': 12.12, 'max_lat': 108.00, 'dataset': '1,2', 'unit': ['km', 'm']})
        self.assertEqual(response.status_code, 400)


class TestCountPointsInPolygonView(TestCase):

    def test_can_load_page(self):
        mp1 = MapPoint(lat=43.0831, lon=73.7846)
        mp1.save()
        response = self.client.get('/api-count/', {'min_lat': 12.12, 'max_lat': 108.00, 'dataset': '1,2',
                                                   'unit': 'km', 'state': 'NY'})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, str(mp1))



from django.test import TestCase, RequestFactory
from django.contrib.auth.models import User
from rest_framework.views import APIView
from gis_csdt.models import DataElement, DataField, Dataset, DatasetNameField, MapElement, MapPoint, MapPolygon, Tag, TagIndiv
from gis_csdt.serializers import AnalyzeAreaNoValuesSerializer, AnalyzeAreaSerializer, CountPointsSerializer, DataPointSerializer, DatasetSerializer, NewTagSerializer, SensorSerializer, TestSerializer
from django.contrib.gis.geos import Polygon, MultiPolygon, Point, fromstr, GEOSGeometry
from django.contrib.auth import get_user_model
from django.http import HttpRequest, QueryDict

User = get_user_model()


class TestDatasetSerializer(TestCase):
    def test_dataset_serializer(self):
        ds = Dataset(name="RPI")
        ds.save()
        data = {'name': 'RPI'}
        serializer = DatasetSerializer(data=data)
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.get_count(ds), 0)
        me = MapElement(dataset=ds)
        me.save()
        self.assertEqual(serializer.get_count(ds), 1)


class TestTestSerializer(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_serializer(self):
        mp = MapPoint.objects.get(pk=1)
        setattr(mp, "field1", "mp_field1")
        setattr(mp, "field2", "mp_field2")
        names = DatasetNameField(field1_name="field1", field2_name="field2", field3_name="field3")
        names.save()
        ds = Dataset(name="ds", names=names)
        ds.save()
        p1 = Polygon(((0, 0), (0, 1), (1, 1), (0, 0)))
        p2 = Polygon(((1, 1), (1, 2), (2, 2), (1, 1)))
        mpoly = MultiPolygon(p1, p2)
        polygon = MapPolygon(lat='42.7302', lon='73.6788', field1=1.0, field2=2.0, mpoly=mpoly, dataset=ds)
        polygon.save()
        me = MapElement(dataset=ds, mappoint=mp, mappolygon=polygon)
        me.save()
        df = DataField(dataset=ds, field_type='I', field_name='int_data', field_en='test')
        df.save()
        element = DataElement(mapelement=me, datafield=df, int_data=23)
        element.save()
        request = RequestFactory().put('/?data=all')
        self.user = User.objects.get(username='test')
        request.user = self.user
        # convert the HTTP Request object to a REST framework Request object
        self.request = APIView().initialize_request(request)
        serializer = TestSerializer(context={'request': self.request})
        expected_address = {"street": "", "city": "Troy", "state": "NY", "zipcode": "", "county": ""}
        self.assertEqual(serializer.get_address(me), expected_address)
        self.assertEqual(serializer.get_data(me)['test'], 23)
        self.assertEqual(serializer.get_geom(me)['type'], 'MultiPolygon')

        tag1 = Tag(dataset=ds, tag='tag1', approved=True, count=1)
        tag1.save()
        tag2 = Tag(dataset=ds, tag='tag2', approved=True, count=1)
        tag2.save()
        tagindiv1 = TagIndiv(tag=tag1, mapelement=me)
        tagindiv1.save()
        tagindiv2 = TagIndiv(tag=tag2, mapelement=me)
        tagindiv2.save()
        self.assertTrue('tag1' in serializer.get_tags(me))
        self.assertTrue('tag2' in serializer.get_tags(me))


class TestCountPointsSerializer(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_serializer(self):
        ds = Dataset.objects.get(pk=1)
        names = DatasetNameField(field1_en='en1', field2_en='en2')
        names.save()
        setattr(ds, "names", names)
        p1 = Polygon(((0, 0), (0, 1), (1, 1), (0, 0)))
        p2 = Polygon(((1, 1), (1, 2), (2, 2), (1, 1)))
        mpoly = MultiPolygon(p1, p2)
        polygon = MapPolygon(lat='42.7302', lon='73.6788', field1=1.0, field2=2.0, mpoly=mpoly, dataset=ds)
        polygon.save()

        df1 = DataField(dataset=ds, field_type='I', field_name='int_data', field_en='test1')
        df1.save()
        df2 = DataField(dataset=ds, field_type='F', field_name='float_data', field_en='test2')
        df2.save()
        df3 = DataField(dataset=ds, field_type='C', field_name='char_data', field_en='test3')
        df3.save()

        tag1 = Tag(dataset=ds, tag="tag1")
        tag1.save()
        tag2 = Tag(dataset=ds, tag="tag2")
        tag2.save()
        tag3 = Tag(dataset=ds, tag="tag3")
        tag3.save()

        element1 = DataElement(mapelement=polygon, datafield=df1, int_data=23)
        element1.save()
        element2 = DataElement(mapelement=polygon, datafield=df2, float_data=2.34)
        element2.save()
        element3 = DataElement(mapelement=polygon, datafield=df3, char_data='abc')
        element3.save()

        request = HttpRequest()
        qdict = QueryDict('', mutable=True)
        qdict.update({'tags': 'tag1,tag2,tag3'})
        qdict.update({'street': '8th St', 'city': 'Troy', 'state': 'NY'})
        qdict.update({'max_lat': '52.5', 'min_lat': '18.9', 'max_lon': '108.1', 'min_lon': '22.1'})
        qdict.update({'radius': 5, 'center': '42.7302,73.6788'})
        request.GET = qdict
        # convert the HTTP Request object to a REST framework Request object
        self.request = APIView().initialize_request(request)
        serializer = CountPointsSerializer(context={'request': self.request})
        count = serializer.get_count(polygon)
        self.assertEqual(count['test1'], 23)
        self.assertEqual(count['test2'], 2.34)
        self.assertEqual(count['en2'], 2.0)


class TestAnalyzeAreaSerializer(TestCase):
    fixtures = ['test_data.json']

    def setUp(self):
        ds = Dataset.objects.get(pk=3)
        self.mp = MapPoint.objects.get(pk=1)
        self.mp.dataset = ds
        self.mp.point = Point(5, 23)
        MapPoint(lat=23.41, lon=98.0, dataset=ds, point=Point(5, 23)).save()
        tag1 = Tag(dataset=ds, tag='tag1', approved=True, count=1)
        tag1.save()
        tag2 = Tag(dataset=ds, tag='tag2', approved=True, count=1)
        tag2.save()
        tagindiv1 = TagIndiv(tag=tag1, mapelement=self.mp)
        tagindiv1.save()
        tagindiv2 = TagIndiv(tag=tag2, mapelement=self.mp)
        tagindiv2.save()
        p1 = GEOSGeometry("SRID=4326;POLYGON ((5 23.00902982868961, 5.004877257506506 23.00781998654152, 5.00844745207902 23.00451469036915, 5.009753953024734 22.99999969967893, 5.008446890194161 22.99548485657674, 5.004876695621637 22.99217985558042, 5 22.99097016102037, 4.995123304378363 22.99217985558042, 4.991553109805839 22.99548485657674, 4.990246046975266 22.99999969967893, 4.99155254792098 23.00451469036915, 4.995122742493493 23.00781998654153, 5 23.00902982868961))")
        p2 = GEOSGeometry("SRID=4326;POLYGON ((5 23.02708945518625, 5.014633459242311 23.02345948568706, 5.025344042308789 23.01354271143227, 5.029261858728946 22.99999729711045, 5.025338985345226 22.98645321108062, 5.014628402277986 22.97653909341024, 5 22.97291045220348, 4.985371597722013 22.97653909341024, 4.974661014654773 22.98645321108062, 4.970738141271054 22.99999729711045, 4.974655957691211 23.01354271143227, 4.985366540757689 23.02345948568706, 5 23.02708945518625))")
        p3 = GEOSGeometry("SRID=4326;POLYGON ((5 23.0451490404854, 5.024391911722986 23.03909835241012, 5.042242881329157 23.02256891874033, 5.048769763397385 22.99999249197385, 5.042228834209113 22.97741975490609, 5.024377864597061 22.96089770063709, 5 22.95485070226389, 4.975622135402939 22.96089770063709, 4.957771165790887 22.97741975490609, 4.951230236602615 22.99999249197385, 4.957757118670842 23.02256891874033, 4.975608088277014 23.03909835241012, 5 23.0451490404854))")
        mpoly1 = MultiPolygon(fromstr(str(p1)))
        polygon1 = MapPolygon(lat='50', lon='22', field1=1.0, field2=2.0, mpoly=mpoly1, dataset=ds, remote_id=10)
        polygon1.save()
        mpoly2 = MultiPolygon(fromstr(str(p2)))
        polygon2 = MapPolygon(lat='12', lon='17', field1=1.0, field2=2.0, mpoly=mpoly2, dataset=ds, remote_id=9)
        polygon2.save()
        mpoly3 = MultiPolygon(fromstr(str(p3)))
        polygon3 = MapPolygon(lat='23', lon='27', field1=1.0, field2=2.0, mpoly=mpoly3, dataset=ds, remote_id=9)
        polygon3.save()


        self.request = HttpRequest()
        qdict = QueryDict('', mutable=True)
        qdict.update({'year': '2010'})
        qdict.update({'year': '2014'})
        qdict.update({'year': '2016'})
        qdict.update({'unit': 'km'})
        self.request.GET = qdict
        self.serializer = AnalyzeAreaSerializer(context={'request': self.request})

    def test_can_get_areaAroundPoint(self):
        data = self.serializer.get_areaAroundPoint(self.mp)
        self.assertEqual(data['point id(s)'], '2')
        self.assertTrue(data.get('1.000000 km'))
        self.assertTrue(data.get('3.000000 km'))
        self.assertTrue(data.get('5.000000 km'))

    def test_can_get_areaAroundPoint2(self):
        data = self.serializer.get_areaAroundPoint2(self.mp)
        self.assertTrue(data.get('1.000000 km'))
        self.assertTrue(data.get('3.000000 km'))
        self.assertTrue(data.get('5.000000 km'))


class TestAnalyzeAreaNoValuesSerializer(TestCase):
    fixtures = ['test_data.json']

    def setUp(self):
        ds = Dataset.objects.get(pk=3)
        self.mp = MapPoint.objects.get(pk=1)
        setattr(self.mp, 'dataset', ds)
        self.mp.point = Point(5, 23)
        MapPoint(lat=23.41, lon=98.0, dataset=ds, point=Point(5, 23), state='NY', city='NYC').save()
        tag1 = Tag(dataset=ds, tag='tag1', approved=True, count=1)
        tag1.save()
        tag2 = Tag(dataset=ds, tag='tag2', approved=True, count=1)
        tag2.save()
        tagindiv1 = TagIndiv(tag=tag1, mapelement=self.mp)
        tagindiv1.save()
        tagindiv2 = TagIndiv(tag=tag2, mapelement=self.mp)
        tagindiv2.save()
        p1 = GEOSGeometry("SRID=4326;POLYGON ((5 23.00902982868961, 5.004877257506506 23.00781998654152, 5.00844745207902 23.00451469036915, 5.009753953024734 22.99999969967893, 5.008446890194161 22.99548485657674, 5.004876695621637 22.99217985558042, 5 22.99097016102037, 4.995123304378363 22.99217985558042, 4.991553109805839 22.99548485657674, 4.990246046975266 22.99999969967893, 4.99155254792098 23.00451469036915, 4.995122742493493 23.00781998654153, 5 23.00902982868961))")
        p2 = GEOSGeometry("SRID=4326;POLYGON ((5 23.02708945518625, 5.014633459242311 23.02345948568706, 5.025344042308789 23.01354271143227, 5.029261858728946 22.99999729711045, 5.025338985345226 22.98645321108062, 5.014628402277986 22.97653909341024, 5 22.97291045220348, 4.985371597722013 22.97653909341024, 4.974661014654773 22.98645321108062, 4.970738141271054 22.99999729711045, 4.974655957691211 23.01354271143227, 4.985366540757689 23.02345948568706, 5 23.02708945518625))")
        p3 = GEOSGeometry("SRID=4326;POLYGON ((5 23.0451490404854, 5.024391911722986 23.03909835241012, 5.042242881329157 23.02256891874033, 5.048769763397385 22.99999249197385, 5.042228834209113 22.97741975490609, 5.024377864597061 22.96089770063709, 5 22.95485070226389, 4.975622135402939 22.96089770063709, 4.957771165790887 22.97741975490609, 4.951230236602615 22.99999249197385, 4.957757118670842 23.02256891874033, 4.975608088277014 23.03909835241012, 5 23.0451490404854))")
        mpoly1 = MultiPolygon(fromstr(str(p1)))
        polygon1 = MapPolygon(lat='50', lon='22', field1=1.0, field2=2.0, mpoly=mpoly1, dataset=ds, remote_id=10)
        polygon1.save()
        mpoly2 = MultiPolygon(fromstr(str(p2)))
        polygon2 = MapPolygon(lat='12', lon='17', field1=1.0, field2=2.0, mpoly=mpoly2, dataset=ds, remote_id=9)
        polygon2.save()
        mpoly3 = MultiPolygon(fromstr(str(p3)))
        polygon3 = MapPolygon(lat='23', lon='27', field1=1.0, field2=2.0, mpoly=mpoly3, dataset=ds, remote_id=9)
        polygon3.save()

        self.request = HttpRequest()
        qdict = QueryDict('', mutable=True)
        qdict.update({'year': '2014'})
        qdict.update({'year': '2016'})
        qdict.update({'unit': 'km'})
        self.request.GET = qdict
        self.serializer = AnalyzeAreaNoValuesSerializer(context={'request': self.request})

    def test_can_get_areaAroundPoint(self):
        data = self.serializer.get_areaAroundPoint(self.mp)
        self.assertEqual(data['1.000000 km'], {'land_area': 1, 'polygons': {u'10': 1.0}})
        self.assertEqual(data['points'][0]['state'], 'NY')
        self.assertEqual(data['points'][0]['city'], 'NYC')


class TestSensorSerializer(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_sensor_model(self):
        request = RequestFactory().get('/?data=all')
        self.user = User.objects.get(username='test')
        request.user = self.user

        attrs = {'name': 'sensor2', 'supplier': 'sss', 'model_number': 123, 'metric': 'm', 'accuracy': 0.001}
        serializer = SensorSerializer(context={'request': request})
        sensor = serializer.create(attrs)
        self.assertEqual(sensor.name, 'sensor2')
        self.assertEqual(sensor.accuracy, 0.001)
        self.assertEqual(sensor.model_number, 123)


class TestDataPointSerializer(TestCase):

    def test_can_create_datapoint_model(self):

        serializer = DataPointSerializer()
        attrs = {'value': 212}
        point = serializer.create(attrs)
        self.assertEqual(point.value, 212)


class TestNewTagSerializer(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_serializer(self):
        ds = Dataset.objects.get(pk=2)
        me = MapElement(name="abc", dataset=ds)
        me.save()
        tag1 = Tag(dataset=ds, tag='test', approved=True)
        tag1.save()
        tag2 = Tag(dataset=ds, tag='test', approved=True)
        tag2.save()
        serializer = NewTagSerializer()
        validated_data = {'mapelement_id': me.id, 'tag': 'test'}
        tagindiv = serializer.create(validated_data)
        self.assertTrue(isinstance(tagindiv, TagIndiv))
        self.assertEqual(tagindiv.tag, tag1)
        self.assertEqual(tagindiv.mapelement, me)
        self.assertEqual(serializer.validate_tag(' test '), 'test')



from django.contrib.gis.db import models
from datetime import datetime
from django.utils.timezone import utc
from django.conf import settings
from django.contrib.auth import get_user_model

import decimal
import json
import urllib


# Field classes for Dataset
class Location(models.Model):
    street_field = models.CharField(max_length=50, default='street',
                                    blank=True)
    city_field = models.CharField(max_length=50, default='city', blank=True)
    state_field = models.CharField(max_length=50, default='state', blank=True)
    zipcode_field = models.CharField(max_length=50, default='zip', blank=True)
    county_field = models.CharField(max_length=50, default='county',
                                    blank=True)

    def __unicode__(self):
        return self.street_field + ", " + self.county_field + ", " \
               + self.city_field + " " + self.state_field \
               + '\n' + self.zipcode_field


class GeoCoordinates(models.Model):
    lat_field = models.CharField(max_length=50, default='latitude', blank=True)
    lon_field = models.CharField(max_length=50, default='longitude',
                                 blank=True)

    def __unicode__(self):
        return "latitude: " + self.lat_field + ", longitude: " + self.lon_field


class DatasetNameField(models.Model):
    field1_en = models.CharField(blank=True, max_length=150)
    field1_name = models.CharField(blank=True, max_length=50)
    field2_en = models.CharField(blank=True, max_length=150)
    field2_name = models.CharField(blank=True, max_length=50)
    field3_en = models.CharField(blank=True, max_length=150)
    field3_name = models.CharField(blank=True, max_length=50)

    def __unicode__(self):
        return self.field1_name + ", " + self.field1_en + ";\n" \
               + self.field2_name + ", " + self.field2_en + ";\n" \
               + self.field3_name + ", " + self.field3_en + ";"


BATCH_SIZE = 5000


class Dataset(models.Model):
    name = models.CharField(max_length=200)
    url = models.URLField(blank=True, max_length=300)
    cached = models.DateTimeField(null=True, blank=True)
    cache_max_age = models.IntegerField('age when cache should be replaced in days',
                                        default=1)
    # field names
    remote_id_field = models.CharField('column name of key field on the remote server',
                                       blank=True, max_length=50, default='id')
    name_field = models.CharField(max_length=50, default='name')
    needs_geocoding = models.BooleanField(default=False)

    location = models.ForeignKey(Location, blank=True, null=True)
    coordinates = models.ForeignKey(GeoCoordinates, blank=True, null=True)
    names = models.ForeignKey(DatasetNameField, blank=True, null=True)

    def __unicode__(self):  # Python 3: def __str__(self):
        return self.name

    # recurses through structure of fields and the json
    # , = level
    # + = concatenation of 2 or more fields.
    def reach_field(self, json_item, location):
        result = ''
        if len(location) > 1:
            for field in location[0]:
                if field in json_item:
                    result += self.reach_field(json_item[field],
                                               location[1:]) + ' '
        elif len(location) == 1:
            for field in location[0]:
                if field in json_item:
                    result += json_item[field].strip() + ' '
        return result.strip()

    '''
    def update_mappoints(self):
        if self.url == '': #this can only be done through manual updates
            return
        if self.should_update():
            self.loop_thru_cache()
        elif self.needs_geocoding:
            for point in MapPoint.objects.filter(dataset_id = self.pk).order_by('remote_id'):
                if not point.geocoded:
                    r = point.geocode()
                    if r['status'] == 'OVER_QUERY_LIMIT':
                        return # ENDS FUNCTION
                    point.save()
            self.needs_geocoding = False
            self.save()

    def loop_thru_cache(self):
        added = 0
        geocoded = 0

        points = MapPoint.objects.filter(dataset_id = self.pk).order_by('remote_id')
        if self.remote_id_field == '':
            plus = ''
        else:
            plus = '?$order=' + self.remote_id_field
        json_in = json.loads(urllib.urlopen(self.url + plus).read())
        if plus == '':
            plus = '?'
        else:
            plus += '&'
        #if json_in['error']:

        #dictionary to hold structure of data in remote dataset
        fields = {}
        fields['remote_id'] = [x.split('+')
                               for x in self.remote_id_field.split(',')]
        fields['name'] = [x.split('+') for x in self.name_field.split(',')]
        fields['lat'] = [x.split('+') for x in self.lat_field.split(',')]
        fields['lon'] = [x.split('+') for x in self.lon_field.split(',')]
        fields['street'] = [x.split('+') for x in self.street_field.split(',')]
        fields['city'] = [x.split('+') for x in self.city_field.split(',')]
        fields['state'] = [x.split('+') for x in self.state_field.split(',')]
        fields['zipcode'] = [x.split('+')
                             for x in self.zipcode_field.split(',')]
        fields['county'] = [x.split('+') for x in self.county_field.split(',')]
        fields['field1'] = [x.split('+') for x in self.field1_name.split(',')]
        fields['field2'] = [x.split('+') for x in self.field2_name.split(',')]
        fields['field3'] = [x.split('+') for x in self.field3_name.split(',')]

        # if there's no remote id,
        # there's no easy way to compare cached with current
        # so just chuck it all and start again
        # if self.remote_id_field == '':
        #	points.delete()
        # err....what about the tags?

        try_geocoding = self.needs_geocoding
        rec_read = len(json_in)
        i = 0
        while len(json_in) > 0:
            for item in json_in:
                if i < len(points) and self.remote_id_field != '':
                    if points[i].remote_id == str(item[self.remote_id_field]):
                        if try_geocoding and not points[i].geocoded:
                            r = points[i].geocode()
                            if r['status'] == 'OVER_QUERY_LIMIT':
                                try_geocoding = False
                                print '--%d geocoded--' %(geocoded)
                            else:
                                geocoded += 1
                        i += 1
                        continue
                    elif points[i].remote_id < str(item[self.remote_id_field]):
                        print points[i].remote_id,
                              str(item[self.remote_id_field])
                        try:
                            print 'Deleting point:', points[i],
                            points[i].delete()
                        except Exception as e:
                            print '...failed to delete point %s:%s' \
                                  %(points[i],e)
                            # not a huge deal if it fails
                            i += 1
                            continue
                        print '...deleted'
                        continue
                new_point = MapPoint(dataset = self, lat=0,lon=0)
                for field in fields:
                    temp = self.reach_field(item, fields[field]).strip()
                    if field in ['lat','lon']:
                        l = len(temp)
                        if l > 18 and temp[0] == '-':
                            temp = temp[:18]
                        elif l > 17:
                            temp = temp[:17]
                        try:
                            temp = decimal.Decimal(temp)
                        except:
                            print 'invalid decimal: %s' %(temp)
                            continue
                    elif len(temp) > MapPoint._meta.get_field(field).max_length:
                        temp = temp[0:MapPoint._meta.get_field(field).max_length]
                    setattr(new_point, field, temp)
                if try_geocoding:
                    r = new_point.geocode()
                    if r['status'] == 'OVER_QUERY_LIMIT':
                        try_geocoding = False
                    else:
                        geocoded += 1
                new_point.save()
                try:
                    new_point.point = Point(float(new_point.lon),
                                            float(new_point.lat))
                    new_point.save()
                except:
                    pass
                added += 1
                if added >= BATCH_SIZE:
                    if try_geocoding:
                        print '--%d added, %d geocoded--' %(added, geocoded)
                    else:
                        print '--%d added--' %(added)
                    return
            json_in = json.loads(urllib.urlopen(self.url + plus
                                 + '$offset=' + str(rec_read)).read())
            rec_read += len(json_in)

        self.cached = datetime.utcnow().replace(tzinfo=utc)
        if try_geocoding: #if still able to geocode, must have completed set
            self.needs_geocoding = False
        print '--%d added, %d geocoded--' %(added,geocoded)
        self.save()
    '''

    def should_update(self):
        if self.cached is None or self.cached == '':
            return True
        since = datetime.utcnow().replace(tzinfo=utc) - self.cached
        if since.days < self.cache_max_age or since.seconds < 60:
            return False
        return True


class MapElement(models.Model):
    dataset = models.ForeignKey(Dataset, blank=True, null=True)
    remote_id = models.CharField(max_length=50, blank=True, null=True)
    name = models.CharField(max_length=150)
    point = models.PointField(srid=4326, blank=True, null=True)
    objects = models.GeoManager()

    def __unicode__(self):
        return self.name

    def polygon_id(self):
        try:
            self.mappolygon
            return self.id
        except Exception:
            return None

    def point_id(self):
        try:
            self.mappoint
            return self.id
        except Exception:
            return None

    # class Meta:
    # abstract = True


# class SensorReading(MapElement)
    # time = DateTimeField()
    # node = name?


class MapPoint(MapElement):
    lat = models.DecimalField(max_digits=18, decimal_places=15)
    lon = models.DecimalField(max_digits=18, decimal_places=15)
    field1 = models.CharField(blank=True, max_length=200)
    field2 = models.CharField(blank=True, max_length=200)
    field3 = models.CharField(blank=True, max_length=200)

    street = models.CharField(blank=True, max_length=200)
    city = models.CharField(blank=True, max_length=100)
    state = models.CharField(blank=True, max_length=2)
    zipcode = models.CharField(blank=True, max_length=5)
    county = models.CharField(blank=True, max_length=75)
    geocoded = models.BooleanField(blank=True, default=False)

    # must be included in children too
    objects = models.GeoManager()

    def geocode(self, unknown_count=0):
        key = settings.GOOGLE_API_KEY
        location = urllib.quote_plus(self.street + ', ' + self.city
                                     + ', ' + self.state + ', ' + self.zipcode)
        request = 'https://maps.googleapis.com/maps/api/geocode/json?address=%s&key=%s&sensor=false' % (location, key)
        # print request
        j = json.loads(urllib.urlopen(request).read())
        if j['status'] == 'OK':
            try:
                self.lat = decimal.Decimal(
                           j['results'][0]['geometry']['location']['lat'])
                self.lon = decimal.Decimal(
                           j['results'][0]['geometry']['location']['lng'])
                self.geocoded = True
            except Exception:
                return {'status': 'conversion_error', 'request': request}
        # this error type means the request can be retried
        elif j['status'] == 'UNKNOWN_ERROR' and unknown_count < 5:
            return self.geocode(unknown_count + 1)
        # want to debug other errors
        assert (j['status'] == 'OK'
                or j['status'] == 'OVER_QUERY_LIMIT'
                or j['status'] == 'ZERO_RESULTS')
        if j['status'] == 'OVER_QUERY_LIMIT':
            print 'Hit Google Maps API daily query limit'
        return {'status': j['status'], 'request': request}


class MapPolygon(MapElement):
    lat = models.CharField(max_length=17)
    lon = models.CharField(max_length=17)
    field1 = models.FloatField()
    field2 = models.FloatField()

    mpoly = models.MultiPolygonField(srid=4326)
    # must be included in children too
    objects = models.GeoManager()


class DataField(models.Model):
    dataset = models.ForeignKey(Dataset, related_name='dataFields')
    field_en = models.CharField(blank=True, max_length=100)
    field_longname = models.CharField(blank=True, max_length=400)
    field_name = models.CharField(blank=True, max_length=50)

    INTEGER = 'I'
    FLOAT = 'F'
    STRING = 'C'
    type_choices = ((INTEGER, 'integer'),
                    (FLOAT, 'floating point'), (STRING, 'string'))
    field_type = models.CharField(max_length=1,
                                  choices=type_choices, default=STRING)

    def __unicode__(self):
        return self.field_en + ', id:' + str(self.id) + \
               ', dataset:' + str(self.dataset_id)


class DataElement(models.Model):
    datafield = models.ForeignKey(DataField, related_name='dataElements')
    mapelement = models.ForeignKey(MapElement)
    denominator = models.ForeignKey('self', null=True)
    error = models.FloatField(null=True)
    int_data = models.IntegerField(null=True)
    float_data = models.FloatField(null=True)
    char_data = models.CharField(blank=True, null=True, max_length=200)

    def __unicode__(self):
        return '[' + self.datafield.field_en + '] for [' + \
               self.mapelement.name + '], id:' + str(self.id) \
               + ', datafield:' + str(self.datafield_id)


class Tag(models.Model):
    dataset = models.ForeignKey(Dataset, related_name='tags')
    tag = models.CharField(max_length=100)
    approved = models.BooleanField(default=False)
    count = models.IntegerField(default=0)

    def __unicode__(self):
        return self.tag

    # not necessary, but would rather have the code centralized
    def increment_count(self, save=True):
        self.count += 1
        if save:
            self.save()

    def recount(self, save=True):
        self.count = TagIndiv.objects.filter(tag=self).filter(
                     mapelement__dataset_id=self.dataset_id).count()
        if save:
            self.save()


class TagIndiv(models.Model):
    tag = models.ForeignKey(Tag)
    mapelement = models.ForeignKey(MapElement)

    def __unicode__(self):
        return self.mapelement.name + ' tagged as "' + self.tag.tag + '"'

    class Meta:
        unique_together = (("tag", "mapelement"),)

    '''
    def save(self, *args, **kwargs):
        exists = self.id is not null
        if not exists:
            matches = TagIndiv.objects.filter(tag = self.tag,
                                              mappoint = self.mappoint,
                                              mappolygon = self.mappolygon)
        if exists or len(matches) == 0:
            super(TagIndiv, self).save(*args, **kwargs)
        elif self.approved and matches.filter(approved=True).count() == 0:
            matches[0].approved = True
            matches[0].save()
    '''


class DataPoint(models.Model):
    value = models.DecimalField(max_digits=30, decimal_places=15)
    time = models.DateTimeField(blank=True, null=True)
    # To add later to tie into django_team
    # team = models.ForeignKey(Team)

    def __unicode__(self):
        return "value: " + str(self.value) + " time: " + str(self.time)


class Sensor(models.Model):
    name = models.CharField(max_length=100, default='name', unique=True)
    supplier = models.CharField(max_length=100, default='supplier')
    model_number = models.CharField(max_length=100, default='model_number')
    metric = models.CharField(max_length=100, default='metric')
    accuracy = models.CharField(max_length=100, default='accuracy')
    user = models.ForeignKey(get_user_model(), blank=True, null=True)
    datapoints = models.ManyToManyField(DataPoint, blank=True)
    mappoint = models.ForeignKey(MapPoint, related_name='points', null=True)

    def __unicode__(self):
        return 'id: ' + str(self.name)


class PhoneNumber(models.Model):
    phone_number = models.BigIntegerField(null=False)
    user = models.ForeignKey(get_user_model(), null=False)



from gis_csdt.models import Dataset, MapElement, MapPoint, Tag, TagIndiv, MapPolygon, DataField, DataElement, Sensor, DataPoint
from gis_csdt.filter_tools import filter_request, neighboring_points, unite_radius_bubbles
from gis_csdt.settings import CENSUS_API_KEY
from rest_framework import serializers
from rest_framework_gis import serializers as gis_serializers
from django.contrib.gis.measure import Distance
from django.contrib.gis.geos import GEOSGeometry
from django.db.models import Sum
from django.http import HttpResponseBadRequest
from django import VERSION as DJANGO_VERSION
import copy
import json
import urllib

CIRCLE_EDGES = 12  # number of edges on polygon estimation of a circle
if DJANGO_VERSION[0] >= 1 and DJANGO_VERSION[1] >= 7:
    def get_function(function_name):
        if function_name[:4] == 'get_':
            return
else:
    def get_function(function_name):
        return function_name

'''class TagSerializer(serializers.ModelSerializer):
    def __init__(self, *args, **kwargs):
        many = kwargs.pop('many', True)
        super(TagSerializer, self).__init__(many=many, *args, **kwargs)

    class Meta:
        model = TagIndiv
        fields = ('mappoint','mappolygon','tag')'''

'''class MapElementIdField(serializers.WritableField):
    def __init__(self, *args,**kwargs):
        self.field = kwargs.pop('field')
        super(MapElementIdField, self).__init__(*args, **kwargs)
    def to_native(self, obj):
        try:
            r = getattr(obj,self.field)
            return r.id
        except ObjectDoesNotExist:
            return None
    def from_native(self, data):
        try:
            return MapElement.objects.get(id=int(data))
        except (ObjectDoesNotExist, ValueError):
            return None'''


class NewTagSerializer(serializers.ModelSerializer):
    # mappolygon = MapElementIdField(required=False,
    #                                source='mapelement_id',
    #                                field='mappolygon')
    mapelement = serializers.IntegerField(source='mapelement_id')
    tag = serializers.CharField()

    class Meta:
        model = TagIndiv
        fields = ('mapelement', 'tag')

    def validate_tag(self, value):
        if ',' in value or type(value) is list:
            raise serializers.ValidationError('No more than one tag should '
                                              'be POSTed at once. Commas '
                                              'are not allowed in tags.')
        return value.strip().lower()

    def create(self, validated_data):
        me = validated_data.pop('mapelement_id')
        try:
            me = MapElement.objects.get(id=me)
        except Exception:
            raise serializers.ValidationError('No MapElement found with ID',
                                              str(me))
        tag_txt = validated_data.pop('tag')
        tags = Tag.objects.filter(dataset=me.dataset, tag=tag_txt)

        len_tags = len(tags)
        if len_tags == 0:
            tag = Tag(dataset=me.dataset, tag=tag_txt)
            tag.save()
        elif len_tags == 1:
            tag = tags[0]
        else:
            approved_tags = tags.filter(approved=True)
            if len(approved_tags) > 0:
                tag = approved_tags[0]
            else:
                tag = tags[0]
        return TagIndiv.objects.create(mapelement=me, tag=tag)


class TagCountSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Tag
        fields = ['dataset', 'tag', 'count']


class DatasetSerializer(serializers.ModelSerializer):
    tags = serializers.SerializerMethodField(get_function('get_tags'))
    count = serializers.SerializerMethodField(get_function('get_count'))

    def get_tags(self, dataset):
        # build nested distinct list
        return Tag.objects.filter(approved=True, dataset=dataset) \
               .order_by('-count').values_list('tag', flat=True)

    def get_count(self, dataset):
        # build nested distinct list
        return MapElement.objects.filter(dataset=dataset).count()

    class Meta:
        model = Dataset
        fields = ('id', 'name', 'cached', 'count', 'tags', 'name_field')


class LocationSerializer(serializers.ModelSerializer):
    street = serializers.CharField(source='location.street')
    city = serializers.CharField(source='location.city')
    state = serializers.CharField(source='location.state')
    zipcode = serializers.CharField(source='location.zipcode')
    county = serializers.CharField(source='location.county')


class GeoCoordinatesSerializer(serializers.ModelSerializer):
    latitude = serializers.DecimalField(source='geocoor.lat', max_digits=18,
                                        decimal_places=15)
    longitude = serializers.DecimalField(source='geocoor.lon', max_digits=18,
                                         decimal_places=15)


class DatasetNameFieldSerializer(serializers.ModelSerializer):
    field1_en = serializers.CharField(source='ds_names.field1_en')
    field2_en = serializers.CharField(source='ds_names.field2_en')
    field3_en = serializers.CharField(source='ds_names.field3_en')
    field1_name = serializers.CharField(source='ds_names.field1_name')
    field2_name = serializers.CharField(source='ds_names.field2_name')
    field3_name = serializers.CharField(source='ds_names.field3_name')


class MapPointSerializer(serializers.HyperlinkedModelSerializer):
    latitude = serializers.DecimalField(source='mappoint.lat', max_digits=18,
                                        decimal_places=15)
    longitude = serializers.DecimalField(source='mappoint.lon', max_digits=18,
                                         decimal_places=15)
    street = serializers.CharField(source='mappoint.street')
    city = serializers.CharField(source='mappoint.city')
    state = serializers.CharField(source='mappoint.state')
    zipcode = serializers.CharField(source='mappoint.zipcode')
    county = serializers.CharField(source='mappoint.county')
    field1 = serializers.CharField(source='mappoint.field1')
    field2 = serializers.CharField(source='mappoint.field2')
    field3 = serializers.CharField(source='mappoint.field3')

    tags = serializers.SerializerMethodField(get_function('get_tags'))

    def get_tags(self, mappoint):
        # build nested distinct list
        return Tag.objects.filter(approved=True,
                                  tagindiv__mapelement=mappoint) \
                          .distinct('id', 'tag') \
                          .values_list('tag', flat=True)

    class Meta:
        # id_field = False
        # geo_field = 'point'
        model = MapPoint
        fields = ('dataset', 'id', 'name', 'latitude', 'longitude', 'street',
                  'city', 'state', 'zipcode', 'county', 'field1',
                  'field2', 'field3', 'tags')


class TestSerializer(gis_serializers.GeoFeatureModelSerializer):
    address = serializers.SerializerMethodField(get_function('get_address'))
    data = serializers.SerializerMethodField(get_function('get_data'))
    tags = serializers.SerializerMethodField(get_function('get_tags'))
    geom = serializers.SerializerMethodField(get_function('get_geom'))

    def get_address(self, mapelement):
        if hasattr(mapelement, 'mappoint'):
            return {'street': mapelement.mappoint.street,
                    'city': mapelement.mappoint.city,
                    'state': mapelement.mappoint.state,
                    'zipcode': mapelement.mappoint.zipcode,
                    'county': mapelement.mappoint.county}
        return {}

    def get_data(self, mapelement):
        data = {}
        child = mapelement.mappoint if hasattr(mapelement, 'mappoint') else mapelement.mappolygon
        dataset = Dataset.objects.get(id=mapelement.dataset_id)
        dataelements = DataElement.objects.filter(mapelement_id=mapelement.id)
        if dataset.names is not None:
            if dataset.names.field1_name != '' and (child.field1 != '' or child.field1 is not None):
                data[dataset.names.field1_en] = child.field1
            if dataset.names.field2_name != '' and (child.field2 != '' or child.field2 is not None):
                data[dataset.names.field2_en] = child.field2
            if hasattr(mapelement, 'mappoint') and dataset.names.field3_name != '' and (child.field3 != '' or child.field3 is not None):
                data[dataset.names.field3_en] = child.field3

        params = self.context.get('request', None).query_params
        if 'data' in params and params['data'] == 'all':
            for de in dataelements:
                for ft, field_name in {DataField.INTEGER: 'int_data', DataField.FLOAT: 'float_data', DataField.STRING: 'char_data'}.iteritems():
                    if de.datafield.field_type == ft:
                        data[de.datafield.field_en] = getattr(de, field_name)

        return data

    def get_tags(self, mapelement):
        # build nested distinct list
        return Tag.objects.filter(approved=True,
                                  tagindiv__mapelement=mapelement) \
               .distinct('tag').values_list('tag', flat=True)

    def get_geom(self, mapelement):
        if hasattr(mapelement, 'mappolygon'):
            return json.loads(GEOSGeometry(mapelement.mappolygon.mpoly).geojson)
        return json.loads(GEOSGeometry(mapelement.point).geojson)

    class Meta:
        id_field = 'id'
        geo_field = 'geom'
        model = MapElement
        fields = ('dataset', 'id', 'name', 'address', 'data', 'tags')


class MapPolygonSerializer(gis_serializers.GeoFeatureModelSerializer):
    latitude = serializers.DecimalField(source='mappolygon.lat',
                                        max_digits=18,
                                        decimal_places=15)
    longitude = serializers.DecimalField(source='mappolygon.lon',
                                         max_digits=18,
                                         decimal_places=15)
    field1 = serializers.CharField(source='mappolygon.field1')
    field2 = serializers.CharField(source='mappolygon.field2')

    mpoly = gis_serializers.GeometryField(source='mappolygon.mpoly')

    tags = serializers.SerializerMethodField(get_function('get_tags'))

    def get_tags(self, mappolygon):
        # build nested distinct list
        return Tag.objects.filter(approved=True,
                                  tagindiv__mapelement=mappolygon) \
               .distinct('id', 'tag').values_list('tag', flat=True)

    class Meta:
        id_field = False
        geo_field = 'mpoly'
        model = MapPolygon
        fields = ('id', 'dataset', 'remote_id', 'name',
                  'latitude', 'longitude', 'field1', 'field2', 'tags')


class CountPointsSerializer(serializers.ModelSerializer):
    polygon_id = serializers.IntegerField(source='remote_id')
    count = serializers.SerializerMethodField(get_function('get_count'))

    class Meta:
        model = MapPolygon
        fields = ('polygon_id', 'count')

    def get_count(self, mappolygon):
        request = self.context.get('request', None)
        params = copy.deepcopy(request.query_params)
        for key in ['max_lat', 'min_lat', 'max_lon', 'min_lon', 'state']:
            try:
                del params[key]
            except Exception:
                pass  # no big deal

        c = {mappolygon.dataset.names.field1_en: mappolygon.mappolygon.field1,
             mappolygon.dataset.names.field2_en: mappolygon.mappolygon.field2}

        datafields = DataField.objects.filter(dataset=mappolygon.dataset)
        # get other data
        for df in datafields:
            data = None
            if df.field_type == DataField.INTEGER:
                element = DataElement.objects.filter(datafield=df) \
                                             .filter(mapelement=mappolygon)
                if element:
                    data = element[0].int_data
            elif df.field_type == DataField.FLOAT:
                element = DataElement.objects.filter(datafield=df) \
                                             .filter(mapelement=mappolygon)
                if element:
                    data = element[0].float_data
            else:
                element = DataElement.objects.filter(datafield=df) \
                                             .filter(mapelement=mappolygon)
                if element:
                    data = element[0].char_data
            if data:
                c[df.field_en] = data

        points = filter_request(params, 'mappoint').filter(point__intersects=mappolygon.mappolygon.mpoly)
        all_tags = None
        if 'tag' in params:
            all_tags = params['tag']
        elif 'tags' in params:
            all_tags = params['tags']
        if all_tags:
            tags = all_tags.split(',')
            if type(tags) is not list:
                tags = [tags]
            all_tags = all_tags.replace(',', ', ')
        else:
            tags = []

        # counts in polygons
        if 'match' not in params or params['match'] != 'all':
            all_tag_filter = points
            for tag in tags:
                try:
                    num = int(tag)
                    tag_obj = Tag.objects.get(num)
                    all_tag_filter = all_tag_filter.filter(tagindiv__tag=tag_obj)
                    c[tag_obj.tag + " count"] = points.filter(tagindiv__tag=tag_obj).count()
                except Exception:
                    all_tag_filter = all_tag_filter.filter(tagindiv__tag__tag=tag)
                    c[tag + " count"] = points.filter(tagindiv__tag__tag=tag).count()
            if len(tags) > 1:
                c[all_tags + " count (match any)"] = points.count()
                c[all_tags + " count (match all)"] = points.count()
        if len(tags) > 1:
            c[all_tags + " count (match all)"] = points.count()

        return c


class AnalyzeAreaSerializer(serializers.ModelSerializer):
    street = serializers.CharField(source='mappoint.street')
    city = serializers.CharField(source='mappoint.city')
    state = serializers.CharField(source='mappoint.state')
    zipcode = serializers.CharField(source='mappoint.zipcode')
    county = serializers.CharField(source='mappoint.county')
    field1 = serializers.CharField(source='mappoint.field1')
    field2 = serializers.CharField(source='mappoint.field2')
    field3 = serializers.CharField(source='mappoint.field3')

    tags = serializers.SerializerMethodField(get_function('get_tags'))
    areaAroundPoint = serializers.SerializerMethodField(get_function('get_areaAroundPoint'))

    class Meta:
        model = MapPoint
        fields = ('street', 'city', 'state', 'zipcode', 'county',
                  'field1', 'field2', 'field3', 'tags', 'areaAroundPoint')

    def get_tags(self, mappoint):
        # build nested distinct list
        return Tag.objects.filter(approved=True,
                                  tagindiv__mapelement=mappoint) \
               .distinct('tag').values('tag')

    def get_areaAroundPoint(self, mappoint):
        request = self.context.get('request', None)

        years = request.GET.getlist('year')
        datasets = Dataset.objects.filter(name__icontains='census')
        if len(years) > 0:
            d = Dataset.objects.none()
            for y in years:
                d = d | datasets.filter(name__contains=y.strip())
            datasets = d
        if len(datasets) == 0:
            return {}
        else:
            dataset_id = datasets[0].id

        # DISTANCES
        distances = request.GET.getlist('distance')
        unit = request.GET.getlist('unit')
        if len(unit) > 1:
            return HttpResponseBadRequest('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m', 'km', 'mi']:
            unit = unit[0]
        else:
            return HttpResponseBadRequest('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1, 3, 5]
            unit = 'km'
        else:
            distances.sort()

        dist_objs = []
        for dist in distances:
            kwargs = {unit: dist}
            dist_objs.append(Distance(**kwargs))

        all_points = neighboring_points(mappoint,
                                        MapPoint.objects
                                        .filter(dataset_id=mappoint.dataset_id),
                                        dist_objs[-1])

        data_sums = {'point id(s)': '', 'view url(s)': []}
        for p in all_points:
            data_sums['point id(s)'] = data_sums['point id(s)'] + ',' + str(p.id)
            data_sums['view url(s)'].append('/around-point/%d/' % (p.id))
        data_sums['view url(s)'] = str(data_sums['view url(s)'])
        data_sums['point id(s)'] = data_sums['point id(s)'].strip(',')
        already_accounted = set()
        boundary = unite_radius_bubbles(all_points, dist_objs)
        for dist in dist_objs:
            if unit == 'm':
                dist_str = '%f m' % (dist.m)
            elif unit == 'km':
                dist_str = '%f km' % (dist.km)
            elif unit == 'mi':
                dist_str = '%f mi' % (dist.mi)
            poly = set(MapPolygon.objects
                       .filter(dataset_id__exact=dataset_id)
                       .filter(mpoly__covers=boundary[dist])
                       .exclude(remote_id__in=already_accounted)
                       .values_list('remote_id', flat=True))
            maybe_polys = MapPolygon.objects.filter(dataset_id__exact=dataset_id) \
                                            .exclude(mpoly__covers=boundary[dist]) \
                                            .exclude(remote_id__in=already_accounted) \
                                            .filter(mpoly__intersects=boundary[dist])
            for polygon in maybe_polys:
                if polygon.mpoly.intersection(boundary[dist]).area > .5 * polygon.mpoly.area:
                    poly.add(polygon.remote_id)
            already_accounted = already_accounted | poly
            data_sums[dist_str] = {}
            data_sums[dist_str]['polygon count'] = len(poly)
            data_sums[dist_str]['land area (m2)'] = sum([int(i)
                                                        for i in MapPolygon.objects
                                                        .filter(dataset_id__exact=dataset_id,
                                                                remote_id__in=poly)
                                                        .values_list('field1', flat=True)])
            if data_sums[dist_str]['polygon count'] > 0:
                data_sums[dist_str]['polygons'] = str(list(poly))
                datafields = DataField.objects.filter(dataset_id__exact=dataset_id).exclude(field_type__exact=DataField.STRING)
                for field in datafields:
                    if field.field_longname not in data_sums[dist_str]:
                        data_sums[dist_str][field.field_longname] = {}
                    if field.field_type == DataField.INTEGER:
                        data = DataElement.objects \
                               .filter(datafield_id=field.id,
                                       mapelement__remote_id__in=poly) \
                               .aggregate(sum=Sum('int_data'),
                                          dsum=Sum('denominator__int_data'))
                        # print data['sum'], data['dsum']
                    elif field.field_type == DataField.FLOAT:
                        data = DataElement.objects \
                               .filter(datafield_id=field.id,
                                       mapelement__remote_id__in=poly) \
                               .aggregate(sum=Sum('float_data'),
                                          dsum=Sum('denominator__float_data'))
                    else:
                        continue
                    data_sums[dist_str][field.field_longname][field.field_en] = data['sum']
                    data_sums[dist_str][field.field_longname]['total'] = data['dsum']

        return data_sums

    def get_areaAroundPoint2(self, mappoint):
        request = self.context.get('request', None)

        years = request.GET.getlist('year')
        year = years[0]
        datasets = Dataset.objects.filter(name__icontains='census')
        if len(years) > 0:
            d = Dataset.objects.none()
            for y in years:
                d = d | datasets.filter(name__contains=y.strip())
            datasets = d
        if len(datasets) == 0:
            return {}
        else:
            dataset_id = datasets[0].id

        # DISTANCES
        distances = request.GET.getlist('distance')
        unit = request.GET.getlist('unit')
        if len(unit) > 1:
            return HttpResponseBadRequest(
                   'No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m', 'km', 'mi']:
            unit = unit[0]
        else:
            return HttpResponseBadRequest('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1, 3, 5]
            unit = 'km'
        else:
            distances.sort()

        dist_objs = []
        for dist in distances:
            kwargs = {unit: dist}
            dist_objs.append(Distance(**kwargs))

        all_points = neighboring_points(mappoint,
                                        MapPoint.objects.filter(dataset=mappoint.dataset_id),
                                        dist_objs[-1])
        # print all_points

        # variables = ['Total Population','Area (km2)','Total (Race)', 'White Only', 'African American', 'Hispanic','Asian/Pacific Islander', 'Native American','Total (Poverty)','below 1.00', 'weighted mean of median household income','Mean Housing Value']
        # variables = {'B02001_001E':{},'B02001_002E':{},'B02009_001E':{},'B03001_001E':{},'B03001_003E':{},'B02011_001E':{}, 'B02012_001E':{},'B02010_001E':{},'B05010_001E':{},'B05010_002E':{},'B19061_001E':{},'B25105_001E':{},'B25077_001E':{},'B25077_001E':{}}
        variables = {'B00001_001E': {}, 'B02001_001E': {}, 'B02001_002E': {},
                     'B02001_003E': {}, 'B02001_004E': {}, 'B02001_005E': {},
                     'B02001_006E': {}, 'B02001_007E': {}, 'B02001_008E': {},
                     'B03001_001E': {}, 'B03001_003E': {}, 'C17002_001E': {},
                     'C17002_002E': {}, 'C17002_003E': {}}
        for v in variables:
            request = 'http://api.census.gov/data/2010/acs5/variables/%s.json?key=%s' \
                      % (v, CENSUS_API_KEY)
            try:
                data = json.loads(urllib.urlopen(request).read())
            except Exception as e:
                print 'variable info for %s failed to load: %s' % (v, request)
                print e
                continue
            variables[v] = data

        data_sums = {'point id(s)': '', 'view url(s)': []}
        for p in all_points:
            data_sums['point id(s)'] = data_sums['point id(s)'] + ',' + str(p.id)
            data_sums['view url(s)'].append('/around-point/%d/' % (p.id))
        data_sums['point id(s)'] = data_sums['point id(s)'].strip(',')
        already_accounted = set()

        boundary = unite_radius_bubbles(all_points, dist_objs)
        for dist in dist_objs:
            if unit == 'm':
                dist_str = '%f m' % (dist.m)
            elif unit == 'km':
                dist_str = '%f km' % (dist.km)
            elif unit == 'mi':
                dist_str = '%f mi' % (dist.mi)
            poly = set(MapPolygon.objects.filter(dataset_id__exact=dataset_id)
                       .filter(mpoly__covers=boundary[dist])
                       .exclude(remote_id__in=already_accounted)
                       .values_list('remote_id', flat=True))
            maybe_polys = MapPolygon.objects.filter(dataset_id__exact=dataset_id) \
                                            .exclude(mpoly__covers=boundary[dist]) \
                                            .exclude(remote_id__in=already_accounted) \
                                            .filter(mpoly__intersects=boundary[dist])
            for polygon in maybe_polys:
                if polygon.mpoly.intersection(boundary[dist]).area > .5 * polygon.mpoly.area:
                    poly.add(polygon.remote_id)
            already_accounted = already_accounted | poly
            data_sums[dist_str] = {}
            data_sums[dist_str]['polygon count'] = len(poly)
            data_sums[dist_str]['land area (m2)'] = sum([int(i) for i in
                                                         MapPolygon.objects
                                                         .filter(dataset_id__exact=dataset_id,
                                                                 remote_id__in=poly)
                                                         .values_list('field1', flat=True)])
            if data_sums[dist_str]['polygon count'] > 0:
                data_sums[dist_str]['polygons'] = str(list(poly))
                if year == '2010':
                    get = ''
                    for e in variables:
                        get = get + ',' + e
                    get = get.strip(',')
                    tracts = {}
                    for e in poly:
                        if e[:2] not in tracts:
                            tracts[e[:2]] = {}
                        if e[2:5] not in tracts[e[:2]]:
                            tracts[e[:2]][e[2:5]] = ''
                        tracts[e[:2]][e[2:5]] = tracts[e[:2]][e[2:5]] + ',' + e[-6:]
                    for state in tracts:
                        for county in tracts[state]:
                            request = 'http://api.census.gov/data/2010/acs5?key=%s&get=%s,NAME&for=tract:%s&in=state:%s,county:%s' \
                                      % (CENSUS_API_KEY, get, tracts[state][county].strip(','), state, county)
                            try:
                                url = urllib.urlopen(request).read()
                                data = json.loads(url)
                            except Exception as e:
                                print e
                                print url
                                print request
                                continue
                            line = data[0]
                            locations = {}
                            for i in range(len(line)):
                                locations[line[i]] = i
                                if line[i] not in data_sums[dist_str] and line[i] in variables:
                                    data_sums[dist_str][line[i]] = copy.deepcopy(variables[line[i]])
                                    data_sums[dist_str][line[i]]['sum'] = 0

                            for line in data[1:]:
                                # get area
                                for v in variables:
                                    try:
                                        data_sums[dist_str][v]['sum'] = data_sums[dist_str][v]['sum'] + int(line[locations[v]])
                                    except Exception:
                                        continue

        return data_sums


class DataPointSerializer(serializers.ModelSerializer):
    value = serializers.DecimalField(max_digits=30, decimal_places=15)
    time = serializers.DateTimeField()

    class Meta:
        model = DataPoint
        fields = ('value', 'time')

    def create(self, attrs, instance=None):
        datapointModel = DataPoint(value=attrs['value'])
        datapointModel.save()
        return datapointModel


class SensorSerializer(serializers.ModelSerializer):
    name = serializers.CharField(max_length=100)
    supplier = serializers.CharField(max_length=100)
    model_number = serializers.CharField(max_length=100)
    metric = serializers.CharField(max_length=100)
    accuracy = serializers.CharField(max_length=100)
    datapoints = DataPointSerializer(many=True, read_only=True)
    mappoint = MapPointSerializer(read_only=True)

    class Meta:
        model = Sensor
        fields = ('name', 'supplier', 'model_number', 'metric', 'accuracy')
        read_only_fields = ('datapoints', 'mappoint')

    def create(self, attrs, instance=None):
        thisUser = self.context['request'].user
        sensorModel = Sensor(name=attrs['name'].strip(),
                             supplier=attrs['supplier'].strip(),
                             model_number=attrs['model_number'],
                             metric=attrs['metric'],
                             accuracy=attrs['accuracy'],
                             user=thisUser)
        sensorModel.save()
        if 'datapoints' in attrs:
            for dp in attrs['datapoints']:
                sensorModel.datapoints.add(DataPoint.objects.get(pk=dp))
                sensorModel.save()
        if 'mappoint' in attrs:
            sensorModel.mappoint = MapPoint.objects.get(pk=attrs['mappoint'])
            sensorModel.save()
        return sensorModel


class AnalyzeAreaNoValuesSerializer(serializers.ModelSerializer):
    point_id = serializers.CharField(source='mappoint.id')
    areaAroundPoint = serializers.SerializerMethodField(get_function('get_areaAroundPoint'))

    class Meta:
        model = MapPoint
        fields = ('point_id', 'areaAroundPoint')

    def get_tags(self, mappoint):
        # build nested distinct list
        return Tag.objects.filter(approved=True,
                                  tagindiv__mapelement=mappoint) \
               .distinct('tag').values('tag')

    def get_areaAroundPoint(self, mappoint):
        request = self.context.get('request', None)
        years = request.GET.getlist('year')
        datasets = Dataset.objects.filter(name__icontains='census')
        if len(years) > 0:
            d = Dataset.objects.none()
            for y in years:
                d = d | datasets.filter(name__contains=y.strip())
            datasets = d
        if len(datasets) == 0:
            return {}
        else:
            dataset_id = datasets[0].id

        # DISTANCES
        distances = request.GET.getlist('distance')
        unit = request.GET.getlist('unit')
        if len(unit) > 1:
            return HttpResponseBadRequest('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m', 'km', 'mi']:
            unit = unit[0]
        else:
            return HttpResponseBadRequest('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1, 3, 5]
            unit = 'km'
        else:
            distances.sort()

        # method is containment or apportionment?
        method = request.GET.getlist('method')
        if len(method) > 0 and method[0] == 'apportionment':
            apportionment = True
        else:
            apportionment = False

        dist_objs = []
        for dist in distances:
            kwargs = {unit: dist}
            dist_objs.append(Distance(**kwargs))

        all_points = neighboring_points(mappoint, MapPoint.objects
                                        .filter(dataset_id=mappoint.dataset_id),
                                        dist_objs[-1])
        # print len(all_points), all_points
        data_sums = {'points': [],
                     'view url': 'http://127.0.0.1:8000/around-point/%d/'
                     % (all_points[0].id)}
        for p in all_points:
            data_sums['points'].append({'id': p.id,
                                        'name': p.name,
                                        'street': p.mappoint.street,
                                        'city': p.mappoint.city,
                                        'state': p.mappoint.state,
                                        'zipcode': p.mappoint.zipcode,
                                        'county': p.mappoint.county,
                                        'field1': p.mappoint.field1,
                                        'field2': p.mappoint.field2,
                                        'field3': p.mappoint.field3,
                                        'tags': Tag.objects.filter(approved=True, tagindiv__mapelement=mappoint).distinct('tag').values('tag')})
        already_accounted = set()
        boundary = unite_radius_bubbles(all_points, dist_objs)
        for dist in dist_objs:
            if unit == 'm':
                dist_str = '%f m' % (dist.m)
            elif unit == 'km':
                dist_str = '%f km' % (dist.km)
            elif unit == 'mi':
                dist_str = '%f mi' % (dist.mi)
            temp_qs = MapPolygon.objects.filter(dataset_id__exact=dataset_id) \
                                        .filter(mpoly__coveredby=boundary[dist]) \
                                        .exclude(remote_id__in=already_accounted) \
                                        .distinct()
            poly = {x: 1. for x in temp_qs.values_list('remote_id', flat=True)}
            try:
                land_area = sum([int(x) for x in temp_qs.values_list('field1', flat=True)])
            except Exception:
                continue
            # print 'poly', poly
            maybe_polys = MapPolygon.objects.filter(dataset_id__exact=dataset_id) \
                                            .exclude(remote_id__in=poly) \
                                            .exclude(remote_id__in=already_accounted) \
                                            .filter(mpoly__intersects=boundary[dist]) \
                                            .distinct()
            # print 'maybes', maybe_polys.values_list('remote_id',flat=True)
            for polygon in maybe_polys:
                intersection_area = polygon.mpoly.intersection(
                                    boundary[dist]).area / polygon.mpoly.area
                if apportionment or intersection_area > .5:
                    poly[polygon.remote_id] = intersection_area \
                                              if apportionment else 1.
                    try:
                        land_area += intersection_area*int(polygon.field1) \
                                     if apportionment else int(polygon.field1)
                    except Exception:
                        continue

            already_accounted = already_accounted | set(poly.keys())
            data_sums[dist_str] = {}
            # data_sums[dist_str]['polygon count'] = len(poly)
            # data_sums[dist_str]['land area (m2)'] = sum([int(i)
            #                                              for i in MapPolygon.objects
            #                                              .filter(dataset_id__exact=dataset_id
            #                                                      ,remote_id__in=poly)
            #                                              .values_list('field1',flat=True)])
            if len(poly) > 0:
                data_sums[dist_str]['polygons'] = poly
                data_sums[dist_str]['land_area'] = land_area
        return data_sums



from time import sleep
from django.test import TestCase
from django.contrib.auth.models import User
from django.contrib.auth import get_user_model
from django.contrib.gis.geos import MultiPolygon, Polygon
from gis_csdt.models import DataPoint, Dataset, GeoCoordinates, Location, MapElement, MapPoint, MapPolygon, Sensor, Tag, TagIndiv
from decimal import Decimal
from datetime import datetime
import pytz

User = get_user_model()


class TestDataset(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_new_dataset(self):
        original_count = Dataset.objects.all().count()
        geo1 = GeoCoordinates(lat_field="43.0831", lon_field="73.7846")
        geo1.save()
        cached = datetime(year=2017, month=10, day=12, hour=12, minute=46, tzinfo=pytz.UTC)
        dataset1 = Dataset(name="Saratoga Spring", cached=cached, coordinates=geo1)
        dataset1.save()
        sleep(1)
        self.assertEqual(Dataset.objects.all().count(), original_count + 1)
        self.assertEqual(dataset1.coordinates.lat_field, "43.0831")
        original_count += 1

        loc2 = Location(city_field="Troy", state_field="NY")
        loc2.save()
        dataset2 = Dataset(name="RPI", location=loc2)
        dataset2.save()
        self.assertEqual(Dataset.objects.all().count(), original_count + 1)
        self.assertEqual(dataset2.location.city_field, "Troy")
        self.assertEqual(dataset2.coordinates, None)
        self.assertEqual(str(dataset2), "RPI")
        self.assertEqual(dataset1.should_update(), True)

    def test_reach_field(self):
        ds = Dataset.objects.get(pk=1)
        result = ds.reach_field({'field2': 'test2', 'field3': 'test3'}, [['field1', 'field2', 'field3']])
        self.assertEqual(result, 'test2 test3')

        result = ds.reach_field({'field2': {'field4': 'test4'}}, [['field1', 'field2', 'field3'], ['field4', 'field5']])
        self.assertEqual(result, 'test4')


class TestMapPoint(TestCase):
    def test_can_create_new_mappoint(self):
        original_count = MapPoint.objects.all().count()
        point = MapPoint(lat=42.7302, lon=73.6788, city='Troy', state='NY')
        point.save()
        sleep(1)
        self.assertEqual(MapPoint.objects.all().count(), original_count + 1)
        self.assertEqual(point.lat, Decimal(42.7302))
        status = point.geocode()['status']
        self.assertTrue(status == 'OK' or status == 'OVER_QUERY_LIMIT' or status == 'ZERO_RESULTS')


class TestDataPoint(TestCase):

    def test_can_create_datapoint(self):
        user = User.objects.create_superuser(username='test',
                                             email='test@test.test',
                                             password='test')
        mappoint = MapPoint(lat=43.0831, lon=73.7846)
        mappoint.save()
        sensor = Sensor.objects.create(name='test', user=user, mappoint=mappoint)
        sensor.save()
        self.assertEqual(str(sensor), 'id: test')
        dp1 = DataPoint(value=25)
        dp1.save()
        sensor.datapoints.add(dp1)
        dp2 = DataPoint(value=20)
        dp2.save()
        sensor.datapoints.add(dp2)
        self.assertEqual(sensor.mappoint.lat, 43.0831)
        self.assertEqual(sensor.datapoints.filter(value=20).count(), 1)
        self.assertEqual(str(dp1), 'value: 25 time: None')


class TestMapElement(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_mapelement(self):
        ds = Dataset.objects.get(pk=1)
        p1 = Polygon(((0, 0), (0, 1), (1, 1), (0, 0)))
        p2 = Polygon(((1, 1), (1, 2), (2, 2), (1, 1)))
        mpoly = MultiPolygon(p1, p2)
        polygon = MapPolygon(lat='42.7302', lon='73.6788', field1=1.0, field2=2.0, mpoly=mpoly, dataset=ds)
        polygon.save()
        mp = MapPoint.objects.get(pk=1)
        original_count = MapElement.objects.all().count()
        me = MapElement(dataset=ds, remote_id='123', name='element', mappolygon=polygon, mappoint=mp)
        me.save()
        self.assertEqual(MapElement.objects.all().count(), original_count + 1)
        self.assertEqual(me.name, 'element')
        self.assertEqual(me.polygon_id(), me.id)
        self.assertEqual(me.point_id(), me.id)


class TestTag(TestCase):
    fixtures = ['test_data.json']

    def test_can_create_tag(self):
        ds = Dataset.objects.get(pk=1)
        original_count = Tag.objects.all().count()
        tag1 = Tag(tag='1', dataset=ds)
        tag1.save()
        tag2 = Tag(tag='2', dataset=ds)
        tag2.save()
        self.assertEqual(Tag.objects.all().count(), original_count + 2)
        self.assertEqual(str(tag1), '1')
        me = MapElement.objects.get(pk=1)
        tagindiv1 = TagIndiv(tag=tag1, mapelement=me)
        tagindiv1.save()
        me1 = MapElement(dataset=ds, name="me1")
        me1.save()
        tagindiv2 = TagIndiv(tag=tag1, mapelement=me1)
        self.assertEqual(str(tagindiv2), 'me1 tagged as "1"')
        tagindiv2.save()
        tag1.recount()
        self.assertEqual(tag1.count, 1)



from rest_framework.exceptions import APIException
from rest_framework.views import exception_handler
from django import VERSION as DJANGO_VERSION


def custom_exception_handler(exc, context=None):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    if DJANGO_VERSION[0] >= 1 and DJANGO_VERSION[1] >= 7:
        response = exception_handler(exc, context)
    else:
        response = exception_handler(exc, context)
    # Now add the HTTP status code to the response.
    if response is not None:
        response.data['status_code'] = response.status_code

    return response


class ServiceUnavailable(APIException):
    status_code = 503
    default_detail = 'Service temporarily unavailable, try again later.'



# -*- coding: utf8 -*-
from rest_framework import generics, permissions, status, viewsets
from rest_framework.settings import api_settings
from rest_framework_csv.renderers import CSVRenderer
from rest_framework.exceptions import ParseError  # , APIException
from django.contrib.gis.db.models import Count
from django.contrib.gis.measure import Distance
from django.http import HttpResponse
from gis_csdt.filter_tools import filter_request, neighboring_points
from gis_csdt.models import DataPoint, Dataset, DatasetNameField, GeoCoordinates, Location, MapElement, MapPoint, MapPolygon, Sensor, Tag, TagIndiv
from gis_csdt.serializers import TagCountSerializer, DatasetNameFieldSerializer, LocationSerializer, GeoCoordinatesSerializer, DatasetSerializer, MapPointSerializer, NewTagSerializer, MapPolygonSerializer, CountPointsSerializer, AnalyzeAreaSerializer, AnalyzeAreaNoValuesSerializer, DataPointSerializer, SensorSerializer
# import csv
from gis_csdt.serializers import TestSerializer
from django.views.decorators.csrf import csrf_exempt


# constants for pagination
PAGINATE_BY_CONST = 100
PAGINATE_BY_PARAM_CONST = 'page_size'
MAX_PAGINATE_BY_CONST = 500
GSM_CHAR_SET = ("@£$¥èéùìòÇ\nØø\rÅåΔ_ΦΓΛΩΠΨΣΘΞ\x1bÆæßÉ !\"#¤%&'()*+,-./0123456789:;<=>?"
                "¡ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÑÜ`¿abcdefghijklmnopqrstuvwxyzäöñüà").decode("utf8")


# classes for pagination
class PaginatedModelViewSet(viewsets.ModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST


class PaginatedReadOnlyModelViewSet(viewsets.ReadOnlyModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST


class NewSensorView(PaginatedModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
    queryset = Sensor.objects.filter()
    serializer_class = SensorSerializer

    # http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.IsAuthenticated]
    # (permissions.IsAuthenticatedOrReadOnly)


class SubmitDataPointView(PaginatedModelViewSet):
    paginate_by = PAGINATE_BY_CONST
    paginate_by_param = PAGINATE_BY_PARAM_CONST
    max_paginate_by = MAX_PAGINATE_BY_CONST
    queryset = DataPoint.objects.all()
    serializer_class = DataPointSerializer

    # http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.IsAuthenticated]
    # (permissions.IsAuthenticatedOrReadOnly)


def GSM7ToInt(string):
    pow = 0
    BASE = 128  # GSM-7 code base
    val = 0
    for char in string:
        val += GSM_CHAR_SET.find(char) * (BASE ** pow)
        pow += 1
    return val


def IntToGSM7(value):
    string = ''
    BASE = 128  # GSM-7 code base
    while value >= BASE:
        string += GSM_CHAR_SET[value % BASE]
        value = int(value / BASE)
    string += GSM_CHAR_SET[value % BASE]
    return string


def DataToGSM7(values):
    string = ''
    BASE = 128  # GSM-7 code base
    MAXSIZE = BASE**2  # 2 bytes of GSM7
    string += IntToGSM7(values.pop(0))
    string += IntToGSM7(values.pop(0))
    for value in values:
        if value > MAXSIZE:
            raise
        if value < BASE:
            string += IntToGSM7(value) + '@'
        else:
            string += IntToGSM7(value)
    return string


@csrf_exempt
def SMSSubmitDataPointView(request):
    DATASIZE = 2  # 2 GSM-7 chars
    data = request.POST.get('Body')
    # phNum = int(request.POST.get('From'))
    # user = PhoneNumber.objects.get(phone_number=phNum).user
    data = [data[i:i+DATASIZE] for i in range(0, len(data), DATASIZE)]
    source = data.pop(0)
    sensor = Sensor.objects.get(pk=GSM7ToInt(source[0]))
    for dataValue in data:
        newData = DataPoint.objects.create(value=GSM7ToInt(dataValue))
        newData.save()
        sensor.datapoints.add(newData)
    return HttpResponse(status=204)


class TestView(PaginatedReadOnlyModelViewSet):
    serializer_class = TestSerializer
    model = MapElement

    def get_queryset(self):
        return filter_request(self.request.query_params, 'mapelement')


class PaginatedCSVRenderer (CSVRenderer):
    results_field = 'results'

    def render(self, data, media_type=None, renderer_context=None):
        if not isinstance(data, list):
            data = data.get(self.results_field, [])
        return super(PaginatedCSVRenderer, self).render(
               data, media_type, renderer_context)


class NewTagViewSet(PaginatedModelViewSet):
    queryset = TagIndiv.objects.filter(tag__approved=True).distinct('tag')
    serializer_class = NewTagSerializer

    # http://www.django-rest-framework.org/api-guide/permissions
    permission_classes = [permissions.AllowAny]
    # (permissions.IsAuthenticatedOrReadOnly)


class TagCountViewSet(PaginatedReadOnlyModelViewSet):
    serializer_class = TagCountSerializer
    model = Tag

    def get_queryset(self):
        return Tag.objects.filter(approved=True).values(
               'dataset', 'tag').annotate(num_tags=Count('id'))


class DatasetViewSet(PaginatedReadOnlyModelViewSet):
    queryset = Dataset.objects.all()
    serializer_class = DatasetSerializer


class LocationViewSet(PaginatedReadOnlyModelViewSet):
    queryset = Location.objects.all()
    serializer_class = LocationSerializer


class GeoCoordinatesViewSet(PaginatedReadOnlyModelViewSet):
    queryset = GeoCoordinates.objects.all()
    serializer_class = GeoCoordinatesSerializer


class DatasetNameFieldViewSet(PaginatedReadOnlyModelViewSet):
    queryset = DatasetNameField.objects.all()
    serializer_class = DatasetNameFieldSerializer


class AddDatasetView(generics.ListCreateAPIView):
    queryset = Dataset.objects.all()
    serializer_class = DatasetSerializer

    def perform_create(self, serializer):
        serializer.save()

    def create(self, request, *args, **kwargs):
        data = request.data
        ds = Dataset.objects.create(**data)
        ds.save()
        return HttpResponse(status=status.HTTP_201_CREATED)


class MapPointViewSet(PaginatedReadOnlyModelViewSet):
    serializer_class = MapPointSerializer
    model = MapPoint

    def get_queryset(self):
        return filter_request(self.request.query_params, 'mappoint')


class AddMapPointView(generics.ListCreateAPIView):
    queryset = MapPoint.objects.all()
    serializer_class = MapPointSerializer

    def perform_create(self, serializer):
        serializer.save()

    def create(self, request, *args, **kwargs):
        data = request.data
        mp = MapPoint.objects.create(**data)
        mp.save()
        return HttpResponse(status=status.HTTP_201_CREATED)


class MapPolygonViewSet(PaginatedReadOnlyModelViewSet):
    serializer_class = MapPolygonSerializer
    model = MapPolygon

    def get_queryset(self):
        return filter_request(self.request.query_params, 'mappolygon')


class CountPointsInPolygonView(PaginatedReadOnlyModelViewSet):
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES + \
                       [PaginatedCSVRenderer]
    serializer_class = CountPointsSerializer
    model = MapPolygon
    # (permissions.IsAuthenticatedOrReadOnly)
    permission_classes = [permissions.AllowAny]

    def get_queryset(self, format=None):
        # split params by which it applies to
        mappolygon_params = {}
        for (param, result) in self.request.query_params.items():
            if param in ['max_lat', 'min_lat', 'max_lon', 'min_lon']:
                mappolygon_params[param] = result
            elif param == 'poly_dataset':
                mappolygon_params['dataset'] = result
        # now get the querysets
        polygons = filter_request(mappolygon_params, 'mappolygon')
        if 'state' in self.request.query_params:
            polygons = polygons.filter(remote_id__startswith=self.request.query_params['state'])
        return polygons


class AnalyzeAreaAroundPointView(PaginatedReadOnlyModelViewSet):
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES \
                       + [PaginatedCSVRenderer]
    serializer_class = AnalyzeAreaSerializer
    model = MapPoint
    # (permissions.IsAuthenticatedOrReadOnly)
    permission_classes = [permissions.AllowAny]

    def get_queryset(self, format=None):
        # split params by which it applies to
        mappoint_params = {}
        for (param, result) in self.request.query_params.items():
            if param in ['max_lat', 'min_lat', 'max_lon', 'min_lon',
                         'dataset', 'tags', 'tag']:
                mappoint_params[param] = result
        # if none of this is specified, this is just too much
        if len(mappoint_params) == 0:
            raise ParseError('Too many results. \
                             Please restrict the mappoints.')
        # now get the queryset
        points = filter_request(mappoint_params, 'mappoint')
        distances = self.request.GET.getlist('distance')
        unit = self.request.GET.getlist('unit')
        if len(unit) > 1:
            raise ParseError('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m', 'km', 'mi']:
            unit = unit[0]
        else:
            raise ParseError('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1, 3, 5]
            unit = 'km'
        else:
            distances.sort()
        kwargs = {unit: distances[-1]}
        take_out = []
        for point in points:
            if point.id in take_out:
                continue
            take_out.extend(neighboring_points(point, points,
                            Distance(**kwargs)).exclude(id=point.id)
                            .values_list('id', flat=True))
        return points.exclude(id__in=take_out)


class AnalyzeAreaAroundPointNoValuesView(PaginatedReadOnlyModelViewSet):
    serializer_class = AnalyzeAreaNoValuesSerializer
    model = MapPoint
    # (permissions.IsAuthenticatedOrReadOnly)
    permission_classes = [permissions.IsAuthenticated]

    def get_queryset(self):
        # split params by which it applies to
        mappoint_params = {}
        for (param, result) in self.request.query_params.items():
            if param in ['max_lat', 'min_lat', 'max_lon', 'min_lon',
                         'dataset', 'tags', 'tag', 'state', 'zipcode']:
                mappoint_params[param] = result
        # if none of this is specified, this is just too much
        if len(mappoint_params) == 0:
            raise ParseError('Too many results. \
                              Please restrict the mappoints.')
        # now get the queryset
        points = filter_request(mappoint_params, 'mappoint')
        distances = self.request.GET.getlist('distance')
        unit = self.request.GET.getlist('unit')
        if len(unit) > 1:
            raise ParseError('No more than one unit may be specified.')
        elif len(unit) == 0:
            unit = 'mi'
        elif unit[0] in ['m', 'km', 'mi']:
            unit = unit[0]
        else:
            raise ParseError('Accepted units: m, km, mi')
        if len(distances) == 0:
            distances = [1, 3, 5]
            unit = 'km'
        else:
            print distances
            distances.sort()
        kwargs = {unit: distances[-1]}
        take_out = []
        for point in points:
            if point.id in take_out:
                continue
            take_out.extend(neighboring_points(point, points,
                            Distance(**kwargs)).exclude(id=point.id)
                            .values_list('id', flat=True))
        return points.exclude(id__in=take_out).distinct()




import os

try:
    from setuptools import setup
    from setuptools import find_packages
except Exception:
    from distutils.core import setup


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def read_requirements(fname):
    f = open(os.path.join(os.path.dirname(__file__), fname))
    return filter(lambda f: f != '', map(lambda f: f.strip(), f.readlines()))


setup(
    zip_safe=False,
    name="gis_csdt",
    version="1.3.2",
    author="Currently SIlly Inventor, formally Kathleen Tully",
    author_email="SillyInventor@gmail.com",
    description="Mapping software to encourage critical thinking",
    keywords="",
    packages=find_packages(),
    long_description=read('README.md'),
    install_requires=read_requirements('requirements.txt'),
    test_suite="dummy",
    include_package_data=True,
)



from django.template import Template, Context
from gis_csdt.settings import GOOGLE_API_KEY
from gis_csdt.models import MapPoint
from gis_csdt.geometry_tools import circle_as_polygon
from gis_csdt.filter_tools import neighboring_points
from django.http import HttpResponse
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.gis.measure import Distance


def get_distances(input):
    try:
        return [float(i.strip()) for i in input.split(',')]
    except:
        return [1, 3, 5]


def AroundPointView(request, mappoint_id=None):
    current_site = get_current_site(request)  # Site.objects.get_current()
    context = {'key': GOOGLE_API_KEY,
               'zoom_level': 12,
               'lat': 1,
               'lon': 45,
               'width': 500,
               'height': 380,
               'root': 'http://' + current_site.domain + '/'}
    if 'lat' in request.GET:
        try:
            context['lat'] = float(request.GET['lat'])
        except:
            pass
    if 'lon' in request.GET:
        try:
            context['lon'] = float(request.GET['lon'])
        except:
            pass

    if 'distances' in request.GET:
        distances = get_distances(request.GET['distances'])
    elif 'distance' in request.GET:
        distances = get_distances(request.GET['distance'])
    else:
        distances = get_distances(None)

    if 'unit' in request.GET and request.GET['unit'] in ['mi', 'm']:
        unit = request.GET['unit']
    else:
        unit = 'km'

    # dataset = 0

    if mappoint_id:
        try:
            mappoint = MapPoint.objects.get(id=mappoint_id)
            if mappoint.point.x != 0:
                context['lon'] = mappoint.point.x
            if mappoint.point.y != 0:
                context['lat'] = mappoint.point.y
            points = [(a.point.y, a.point.x, a.name)
                      for a in neighboring_points(mappoint,
                                                  MapPoint.objects.filter(dataset=mappoint.dataset_id),
                                                  Distance(km=distances[-1]))]
            # dataset = mappoint.dataset_id
        except Exception:
            points = [(context['lat'], context['lon'], "")]
    else:
        points = [(context['lat'], context['lon'], "")]
    print context['root']
    if context['root'] == 'http://example.com/':
        context['root'] = 'http://127.0.0.1:8000/'
    context['circle'] = ''

    for d in distances:
        text = ''
        dist_kwargs = {unit: d}
        dist = Distance(**dist_kwargs)
        geometry = circle_as_polygon(lat=points[0][0], lon=points[0][1], distance=dist)
        for p in points[1:]:
            geometry = geometry.union(circle_as_polygon(lat=p[0], lon=p[1], distance=dist))
        for poly in geometry:
            text = ''
            for ring in poly:
                if type(ring) is tuple:
                    text = text + ', new google.maps.LatLng(%f, %f)\n' % (ring[1], ring[0])
                else:
                    for coord in ring:
                        text = text + ', new google.maps.LatLng(%f, %f)\n' % (coord[1], coord[0])
            context['circle'] = context['circle'] + ',['+text.strip(',')+']'
    context['circle'] = '[' + context['circle'].strip(',') + ']'
    context['points'] = str(['new google.maps.LatLng(%f,%f)' % (a[0], a[1]) for a in points]).replace("'", "")
    context['names'] = str([a[2] for a in points]).replace("u'", "'")
    context = Context(context)
    template = """<!DOCTYPE HTML>
    <script src="http://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
    <script src="http://maps.googleapis.com/maps/api/js?key={{ key }}&sensor=false">
    </script>

    <script>
    var map;
    function initialize() {
        function add_polygons(){
          var polygon = new google.maps.Polygon({
            paths: {{ circle }},
            strokeColor: '#0033FF',
            strokeOpacity: 0.8,
            strokeWeight: 2,
            fillColor: '#FF0000',
            fillOpacity: 0.35
          });
          polygon.setMap(map);
        }
        function set_height(){
            var h = $(window).height();
        $('#googleMap').css('height', h);
        }
        $(window).resize(set_height());
        set_height();
        // Create a simple map.
        map = new google.maps.Map(document.getElementById('googleMap'), {
            zoom: {{ zoom_level }},
            center: {lat: {{ lat }}, lng:{{ lon }}}
        });
          add_polygons();
      // Load a GeoJSON from the same server as our demo.
      google.maps.event.addListener(map, 'bounds_changed', function() {
          load_polys("{{ root }}api-poly/?max_lat=" + map.getBounds().getNorthEast().lat() + "&max_lon=" + map.getBounds().getNorthEast().lng() + "&min_lat=" + map.getBounds().getSouthWest().lat() + "&min_lon="+ map.getBounds().getSouthWest().lng());
          add_markers("{{ root }}api-mp/?dataset=" + {{ dataset }} +"&max_lat=" + map.getBounds().getNorthEast().lat() + "&max_lon=" + map.getBounds().getNorthEast().lng() + "&min_lat=" + map.getBounds().getSouthWest().lat() + "&min_lon="+ map.getBounds().getSouthWest().lng());


        function add_markers(json_next) {
            $.getJSON(json_next)
                .done(function(data){
                      for (var i=0;i<data.results.length;i++){
                          var marker = new google.maps.Marker({
                            position: new google.maps.LatLng(data.results[i].latitude, data.results[i].longitude),
                            title: data.results[i].name
                        });
                        marker.setMap(map);
                      }

                    json_next = data.next;
                    if (data.next != null){
                        load_polys(data.next);
                    }

                });
              var centers = {{ points }};
              var names = {{ names|safe }};
              for (var i=0;i<centers.length;i++){
                  var marker = new google.maps.Marker({
                    position: centers[i],
                    title: names[i]
                });
                marker.setMap(map);
              }
        }

        function load_polys(json_next) {
            $.getJSON(json_next)
            .done(function(data){
                map.data.addGeoJson(data.results);
                json_next = data.next;
                if (data.next != null){
                    load_polys(data.next);
                }

            });
        }
        var featureStyle = {
    fillOpacity: 0,
    strokeWeight: 1
  }
  map.data.setStyle(featureStyle);

      });

    }

    $( document ).ready(google.maps.event.addDomListener(window, 'load', initialize));

    </script>
    <div id="googleMap" style="width:100%;height:0px;"></div>
"""
    template = Template(template)
    return HttpResponse(template.render(context))
